//----------------------------------------------------------------------------------
//	FILE:			Solar-Main.C
//
//	Description:	Power Library Test Bench 
//
//	Version: 		1.0
//
//  Target:  		TMS320F2803x(PiccoloB)
//
//----------------------------------------------------------------------------------//  
//Copyright Texas Instruments 2004-2009
//----------------------------------------------------------------------------------
//  Revision History:
//----------------------------------------------------------------------------------
//  Date	  | Description / Status
//----------------------------------------------------------------------------------
// 29 OCT 2010 - Vieri
//----------------------------------------------------------------------------------
//
// PLEASE READ - Useful notes about this Project

// Although this project is made up of several files, the most important ones are:
//	 "{ProjectName}-Main.C"	- this file
//		- Application Initialization, Peripheral config,
//		- Application management
//		- Slower background code loops and Task scheduling
//	 "{ProjectName}-DevInit_F28xxx.C
//		- Device Initialization, e.g. Clock, PLL, WD, GPIO mapping
//		- Peripheral clock enables
//		- DevInit file will differ per each F28xxx device series, e.g. F280x, F2833x,
//	 "{ProjectName}-DPL-ISR.asm
//		- Assembly level library Macros and any cycle critical functions are found here
//	 "{ProjectName}-Settings.h"
//		- Global defines (settings) project selections are found here
//		- This file is referenced by both C and ASM files.

// Code is made up of sections, e.g. "FUNCTION PROTOTYPES", "VARIABLE DECLARATIONS" ,..etc
//	each section has FRAMEWORK and USER areas.
//  FRAMEWORK areas provide useful ready made "infrastructure" code which for the most part
//	does not need modification, e.g. Task scheduling, ISR call, GUI interface support,...etc
//  USER areas have functional example code which can be modified by USER to fit their appl.
//
// Code can be compiled with various build options (Incremental Builds IBx), these
//  options are selected in file "{ProjectName}-Settings.h".  Note: "Rebuild All" compile
//  tool bar button must be used if this file is modified.
//----------------------------------------------------------------------------------
#include "SolarHv_DCAC-Settings.h"
#include "PeripheralHeaderIncludes.h"
#include "DSP2803x_EPWM_defines.h"		
		
#include "DPlib.h"	
#include "IQmathLib.h"

#ifdef FLASH
#define DLOG_SIZE 400	// Uncomment for FLASH configuration only
#else
#define DLOG_SIZE 150
#endif
//#define DLOG_SIZE 32	// Uncomment for RAM configuration only

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// FUNCTION PROTOTYPES
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// -------------------------------- FRAMEWORK --------------------------------------
// State Machine function prototypes
//----------------------------------------------------------------------------------
// Alpha states

// adding protection features structure for variables here

enum ProP{Start, Protect, Locked};
struct Pvarsp{
    Uint16 UnderVoltageTrip;    // Under voltage trip point
    Uint16 OverVoltageTrip;     // Over Voltage trip point
    Uint16 OverCurrentTrip;     // Over Current trip point
    Uint16 Hysteresis;          // Under voltage hysteresis
    Uint16 Volts;               // volts signal
    int16 Amps;                 // amps signal
    Uint16 AmpsMax;             // save max value
    Uint16 AmpsTimer;           // use to decay the amps max value
    Uint16 LoopRate;            // use to test the sample rate
    Uint16 FaultLock:1;         // Set when a trip occurs.  Power cycle to clear
    Uint16 ForceAdvance:1;      // Force advance to normal mode to stop red led blinking
    Uint32 DelayTimer;          // Wait before enforcement
    Uint16 AmpOut;              // save the amp trip value
    Uint16 PWMonSyncFlag:1;     // turn on PWM at sine zero crossing
    Uint16 SinePolarity:1;      // polarity of the sine sample
    Uint16 ACzero;              // get the AC zero value
    Uint16 DACzero;             // calculate the DAC zero
    Uint16 DAChighSide;         // DAC high side trip in amps
    Uint16 DAClowSide;          // DAC low side trip in amps
    Uint16 UnderVoltEnable:1;   // Enable the Undervoltage function
    enum ProP Pstate;
} Pvars;

//

void ProtectionPolling(void);    // prototype for the new Protection feature

void A0(void);	//state A0
void B0(void);	//state B0
void C0(void);	//state C0

// A branch states
void A1(void);	//state A1
void A2(void);	//state A2
void A3(void);	//state A3
void A4(void);	//state A4

// B branch states
void B1(void);	//state B1
void B2(void);	//state B2
void B3(void);	//state B3
void B4(void);	//state B4

// C branch states
void C1(void);	//state C1
void C2(void);	//state C2
void C3(void);	//state C3
void C4(void);	//state C4

// Variable declarations
void (*Alpha_State_Ptr)(void);	// Base States pointer
void (*A_Task_Ptr)(void);		// State pointer A branch
void (*B_Task_Ptr)(void);		// State pointer B branch
void (*C_Task_Ptr)(void);		// State pointer C branch


//----------------------------------------------------------------------------------

void DeviceInit(void);
extern void SCIA_Init();
extern void SerialHostComms();
#ifdef FLASH		
	void InitFlash();
#endif
void MemCopy();
//extern void SerialHostComms();
//extern void 	SCIA_Init();

//-------------------------------- DPLIB --------------------------------------------
void ADC_SOC_CNF(Uint8 ChSel[], Uint8 Trigsel[], Uint8 ACQPS[], Uint8 IntChSel, Uint8 mode);
void PWM_SolarInv_CNF(int16 n, int16 Period);

void InitAdc(void);
void AdcOffsetSelfCal(void);
void MoveCurrentLimtBar(void);
extern void TripZoneInit(void);

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - GENERAL
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- FRAMEWORK --------------------------------------


static int LoopCounter = 0;

int16	VTimer0[4];					// Virtual Timers slaved off CPU Timer 0
int16	VTimer1[4];					// Virtual Timers slaved off CPU Timer 1
int16	VTimer2[4];					// Virtual Timers slaved off CPU Timer 2
int16	SerialCommsTimer;
int16	CommsOKflg;
int16	HRmode;
int16	BlinkStatePtr, LED_TaskPtr;
int16   wTaskA_Timer_Cnt;
int16   wTaskB_Timer_Cnt;
int16   wTaskC_Timer_Cnt;

// Used for running BackGround in flash, and ISR in RAM
extern Uint16 *RamfuncsLoadStart, *RamfuncsLoadEnd, *RamfuncsRunStart;
extern Uint16 Cla1funcsLoadStart, Cla1funcsLoadEnd, Cla1funcsRunStart;

// Used for ADC Configuration 
Uint8 	ChSel[16] =   {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
Uint8		TrigSel[16] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
Uint8     ACQPS[16] =   {7,7,7,7,7,7,7,7,7,7,7,7,7,7,7,7};
// ---------------------------------- USER -----------------------------------------
// ---------------------------- DPLIB Net Pointers ---------------------------------
// Declare net pointers that are used to connect the DP Lib Macros  here 

// ADCDRV_4ch 
extern volatile long *ADCDRV_5ch_RltPtrA; 	// instance #0
extern volatile long *ADCDRV_5ch_RltPtrB; 	// instance #1
extern volatile long *ADCDRV_5ch_RltPtrC; 	// instance #2
extern volatile long *ADCDRV_5ch_RltPtrD; 	// instance #3
extern volatile long *ADCDRV_5ch_RltPtrE; 	// instance #4

// CONTROL_2P2Z
extern volatile long *CNTL_2P2Z_Ref1;	// instance #1
extern volatile long *CNTL_2P2Z_Out1;	// instance #1
extern volatile long *CNTL_2P2Z_Fdbk1;	// instance #1
extern volatile long *CNTL_2P2Z_Coef1; 	// instance #1
extern volatile long CNTL_2P2Z_DBUFF1[5];

// CONTROL_2P2Z
extern volatile long *CNTL_2P2Z_Ref2;	// instance #1
extern volatile long *CNTL_2P2Z_Out2;	// instance #1
extern volatile long *CNTL_2P2Z_Fdbk2;	// instance #1
extern volatile long *CNTL_2P2Z_Coef2; 	// instance #1
extern volatile long CNTL_2P2Z_DBUFF2[5];

// CONTROL_2P2Z
extern volatile long *CNTL_2P2Z_Ref3;	// instance #1
extern volatile long *CNTL_2P2Z_Out3;	// instance #1
extern volatile long *CNTL_2P2Z_Fdbk3;	// instance #1
extern volatile long *CNTL_2P2Z_Coef3; 	// instance #1
extern volatile long CNTL_2P2Z_DBUFF3[5];

// CONTROL_2P2Z
extern volatile long *CNTL_2P2Z_Ref4;	// instance #1
extern volatile long *CNTL_2P2Z_Out4;	// instance #1
extern volatile long *CNTL_2P2Z_Fdbk4;	// instance #1
extern volatile long *CNTL_2P2Z_Coef4; 	// instance #1
extern volatile long CNTL_2P2Z_DBUFF4[5];

//INV_ICMD
extern volatile long *INV_ICMD_Vcmd1;
extern volatile long *INV_ICMD_Vac1; 		 
extern volatile long *INV_ICMD_Out1;		 
extern volatile long *INV_ICMD_Comp1;
extern volatile long *INV_ICMD_Out_Max1;
extern volatile long *INV_ICMD_Out_Min1;

//PWMDRV_FBINV
extern volatile long *PWMDRV_Duty;	   
extern volatile long *PWMDRV_Temp;       
extern volatile long *PWMDRV_Ratio;    

//GEN_SIN_COS
extern volatile long *GEN_SIN_COS_Sin_0;			 
extern volatile long *GEN_SIN_COS_Sin_1;			 
extern volatile long *GEN_SIN_COS_Cos_0;		 
extern volatile long *GEN_SIN_COS_Cos_1;			 
extern volatile long *GEN_SIN_COS_Max;  	 
extern volatile long *GEN_SIN_COS_Min;  	 
extern volatile long *GEN_SIN_COS_Ws;		 
extern volatile long *GEN_SIN_COS_Ts;	 	 	  

// DLOG_4CH
extern volatile long *DLOG_4ch_i1Ptr;
extern volatile long *DLOG_4ch_i2Ptr;
extern volatile long *DLOG_4ch_i3Ptr;
extern volatile long *DLOG_4ch_i4Ptr;
extern volatile int16 *DLOG_4ch_buff1Ptr;
extern volatile int16 *DLOG_4ch_buff2Ptr;
extern volatile int16 *DLOG_4ch_buff3Ptr;
extern volatile int16 *DLOG_4ch_buff4Ptr;
extern volatile long DLOG_4ch_TrigVal;
extern volatile long DLOG_4ch_SkipCnt;
extern volatile int16 DLOG_4ch_PreScalar;
extern volatile int16 DLOG_4ch_Size; 

// ---------------------------- DPLIB Variables ---------------------------------
// Declare the net variables being used by the DP Lib Macro here 

volatile long dwInv_Current_1;	//Inv current 1
volatile long dwInv_Current_2;
volatile long dwInv_Voltage_1;
volatile long dwInv_Voltage;
volatile long dwMid_Ref_Volt;
volatile long dwPLCResr;
volatile long dwInv_Frequency,dwInv_Frequency_0;

volatile long dwBus_Voltage_Ref,dwBus_Voltage_Fbk,dwBus_Voltage_Loop_Out_1,dwBus_Voltage_Loop_Out;
volatile long dwInv_Curr_Ref;
volatile long dwInv_Curr_LoopOut;
volatile long dwDuty_Cal_Out;
volatile long dwPWM_Cal_Ratio;
volatile long dwDuty_Temp;
volatile long dwSine_Ref,dwFdFwd_Comp,dwInvCurrRef_Max,dwInvCurrRef_Min;
volatile long dwDuty_Cal_Max,dwDuty_Cal_Min; 

volatile long dwPll_Sin_0,dwPll_Sin_Out,dwPll_Cos_0,dwPll_Cos_out,dwPllCon_Out;
volatile long dwPll_Sin_Cos_Max,dwPll_Sin_Cos_Min;
volatile long dwPll_Trace_Freq,dwPll_Sample_Time;	
long dwRef_Sin_Cal_Ratio,dwRef_Sin_Out,dwPhase_Error_Out,dwPhase_Err_Notch_Out;
long dwPhase_Err_Max,dwPhase_Err_Min;
volatile long dwZero = 0;
long dwPll_Trace_Freq_Ratio = _IQ20(628.31852);		//2*100*pi
long dwPhase_Err_Offset;
unsigned long dwUtility_Freq_Cnt,dwUtility_Freq_Cnt_0,dwUtility_Period;
unsigned long dwFreqTemp;
int16  wIOTest = 0;

//Calculation variables
int32 dwInvVoltSumTemp,dwInvVoltSum;
int32 dwInvCurrSumTemp,dwInvCurrSum;
int32 dwDcBusSumTemp,dwDcBusSum;
int32 dwActivePowerSumTemp,dwActivePowerSum;
int16 wSumCnt,wSumCntTemp,wLineCycleFlag,wNoZeroCrossCnt;
int32 dwOneThirdRatio;
int32 dwInv_Current_Ratio,dwInv_Voltage_Ratio,dwBus_Voltage_Ratio,dwActivePower_Ratio;
int16 wCyclePeriodCnt,wCalculationCnt,wSysStatusCheckCnt;	

//GUI interface variables
int32 Gui_Bus_Volt_Ref;
int16 Gui_wInvVoltRms;
int16 Gui_wInvCurrRms;
int16 Gui_wDcBusVoltAvg;
int16 Gui_wActivePower;
int16 Gui_wInvFreq;
int16 Gui_wPanelVolt,Gui_wPanelCurrent,Gui_wBoostOutputVolt,Gui_wPanelOutputPower;
int16 Gui_wInvVoltOverChkCnt,Gui_wInvVoltUnderChkCnt,Gui_wInvVolt_High_Limit,Gui_wInvVolt_Low_Limit;
int16 Gui_wInvCurrChkCnt,Gui_wInvCurr_High_Limit;
int16 Gui_wBusVoltOverChkCnt,Gui_wBusVoltUnderChkCnt,Gui_wBusVolt_High_Limit,Gui_wBusVolt_Low_Limit;
int16 Gui_wBusVoltRushChkCnt,Gui_wBusVolt_Rush_Limit;
int16 Gui_wFreqOverChkCnt,Gui_wFreqUnderChkCnt,Gui_wFreq_High_Limit,Gui_wFreq_Low_Limit;
int16 Gui_wShortCircuitChkCnt;
int16 Gui_wFaultCode;

int16 Gui_wUtility_Setting;
int16 Gui_Utility_Freq;
int16 Gui_Utility_Volt;
int16 Gui_Utility_Setting_Enable;

int16 Gui_wLoadDefault_Enable,Gui_wSetting_Enable;
int16 Gui_wInvVolt_High,Gui_wInvVolt_Low,Gui_wFreq_High,Gui_wFreq_Low;
int16 Gui_wSysMode;

int16 Gui_wTurnOn,Gui_wTurnOff;
int16 Gui_wGridTie_Status,Gui_wMPPT_Status;
int16 Gui_GridTie_Enable;
int16 Gui_DC_Board_Connect;

int16 Gui_wLLCOutputVolt;
int16 Gui_wMPPT_OnOff;


int16 Gui_wSampleRate,Gui_wSampleLength;
int16 Gui_CH1_ID,Gui_CH2_ID,Gui_CH3_ID,Gui_CH4_ID;
int16 Gui_wLogEnable,Gui_wLogStatus;



#pragma DATA_SECTION(CNTL_2P2Z_CoefStruct1, "CNTL_2P2Z_Coef"); 
#pragma DATA_SECTION(CNTL_2P2Z_CoefStruct2, "CNTL_2P2Z_Coef"); 
#pragma DATA_SECTION(CNTL_2P2Z_CoefStruct3, "CNTL_2P2Z_Coef"); 
#pragma DATA_SECTION(CNTL_2P2Z_CoefStruct4, "CNTL_2P2Z_Coef"); 

struct CNTL_2P2Z_CoefStruct CNTL_2P2Z_CoefStruct1;
struct CNTL_2P2Z_CoefStruct CNTL_2P2Z_CoefStruct2;
struct CNTL_2P2Z_CoefStruct CNTL_2P2Z_CoefStruct3;
struct CNTL_2P2Z_CoefStruct CNTL_2P2Z_CoefStruct4;

#pragma DATA_SECTION(DBUFF1,"DLOG_BUFF");
#pragma DATA_SECTION(DBUFF2,"DLOG_BUFF");
#pragma DATA_SECTION(DBUFF3,"DLOG_BUFF");
#pragma DATA_SECTION(DBUFF4,"DLOG_BUFF");
int16 DBUFF1[DLOG_SIZE];
int16 DBUFF2[DLOG_SIZE];
int16 DBUFF3[DLOG_SIZE];
int16 DBUFF4[DLOG_SIZE];
int16 wDataLogEnable;

#ifdef FLASH	
int* DLOG_point;
int* DLOG_point_vac;  

#endif

int32 dwSinIndexPointer = 0;
int32 dwDeltaTemp = 0;
int16 wSysMode;				//Running mode of the system
int16 wCycleFlag;
int16 wTestTempCnt;
int16 wSinAmp;
int16 wSinAmp_I;
int16 wBus_V;
int16 wStartCheckCnt;
long dwTestTemp;
int16 wTurnOffByButton;
int16 wGridTieEnable;
int16 wSendTurnOnCmd;
int16 wSendTurnOffCmd;
int16 wNoSciRevIntCnt;
int16 wVoltageLoopEnable;
int32 dwVoltageLoopOutConst;
int16 wGPINCheckCnt;

union FSysFlags{
	
	Uint16	word;
	
	struct {
		Uint16	FwTurnOn:1;
		Uint16	FwTurnOff:1;
		
		Uint16  FwWarning:1;
		Uint16  FwFault:1;
		
		Uint16  FwClearFaultFlag:1;
		Uint16  FwOpRlyStatus:1;
		Uint16  FwPwmStatus:1;
		
		Uint16  FwAutoStartOnEn:1;
		Uint16  FwTurnOnConditionOk:1;
		Uint16  FwAutoStart:1;
		
		Uint16   resv:6;
		}BIT;
}FSuperFlag;
//union FSysFlags FSuperFlag;

//#ifdef FLASH
volatile long *dwGraphIdTab[16] = {
	0,
	&dwInv_Current_1,			&dwInv_Voltage,			&dwBus_Voltage_Fbk,
	&dwBus_Voltage_Loop_Out,	&dwInv_Curr_LoopOut,	&dwInv_Curr_Ref,
	&dwPll_Trace_Freq,			&dwPhase_Error_Out, 	&dwPllCon_Out,
	&dwPhase_Err_Notch_Out,		0,						0,
	&dwRef_Sin_Out,				&dwPll_Sin_Out,			&dwPll_Cos_out};//15 dwLine_Curr_LoopOut_T 1 dwDuty_Cal_Out_T 0 
//#endif



//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
//MACROS DECLARATIONS
#define		mP_PWM_OFF()		{EALLOW; EPwm1Regs.TZFRC.bit.OST = 1; EDIS;}
#define		mP_PWM_ON()			{EALLOW; EPwm1Regs.TZCLR.bit.OST = 1; EDIS;}
#define		mN_PWM_OFF()		{EALLOW; EPwm2Regs.TZFRC.bit.OST = 1; EDIS;}
#define		mN_PWM_ON()			{EALLOW; EPwm2Regs.TZCLR.bit.OST = 1; EDIS;}
#define		mALL_PWM_ON()		{EALLOW; \
								 EPwm1Regs.TZCLR.bit.OST = 1; \
								 EPwm2Regs.TZCLR.bit.OST = 1;\
								 FSuperFlag.BIT.FwPwmStatus = 1;\
								 EDIS;}
#define		mALL_PWM_OFF()		{EALLOW; \
								 EPwm1Regs.TZFRC.bit.OST = 1; \
								 EPwm2Regs.TZFRC.bit.OST = 1;\
								 FSuperFlag.BIT.FwPwmStatus = 0;\
								 EDIS;}
#define		mFREE_RUN_ON()		{GpioDataRegs.GPACLEAR.bit.GPIO4 = 1;\
										asm(" nop");asm(" nop");\
										asm(" nop");asm(" nop");\
										asm(" nop");asm(" nop");}
#define		mFREE_RUN_OFF()		{GpioDataRegs.GPASET.bit.GPIO4 = 1;\
										asm(" nop");asm(" nop");\
										asm(" nop");asm(" nop");\
										asm(" nop");asm(" nop");}
										
#define		mGPIN_STATUS()		GpioDataRegs.GPADAT.bit.GPIO6

#define		mOPRLY_ON()			{GpioDataRegs.GPADAT.bit.GPIO7 = 1;FSuperFlag.BIT.FwOpRlyStatus = 1;}
#define		mOPRLY_OFF()		{GpioDataRegs.GPADAT.bit.GPIO7 = 0;FSuperFlag.BIT.FwOpRlyStatus = 0;}

#if TRANSPHORM_TDPVXXXXE0C1
#define 	mSWON				1	// No hardware button present
#else
#define		mSWON				GpioDataRegs.GPBDAT.bit.GPIO41
#endif

#define     mModeLed_ON()		{GpioDataRegs.GPADAT.bit.GPIO31 = 0;}
#define		mModeLed_OFF()		{GpioDataRegs.GPADAT.bit.GPIO31 = 1;}
#define		mFaultLed_ON()		{GpioDataRegs.GPBDAT.bit.GPIO34 = 0;}
#define		mFaultLed_OFF()		{GpioDataRegs.GPBDAT.bit.GPIO34 = 1;}

#define		cPowerOnMode		0
#define     cStandByMode		1
#define		cSoftStartMode		2
#define		cNormalInvMode		3
#define		cFaultMode			4

#define		cTurnOn				1
#define		cTurnOff			2

//DC bus voltage Q24  Vdc*10*Q24/(3020*3.3)
#define		cBus500V			_IQ24(0.5017)
#define		cBus400V			_IQ24(0.4014)
#define		cBus380V			_IQ24(0.381)
#define		cBus300V			_IQ24(0.301)
#define		cBus200V			_IQ24(0.2007)
#define		cBus100V			_IQ24(0.1003)
#define		cBus50V				_IQ24(0.0502)
#define     cBus20V				_IQ24(0.02)
#define		cBus10V				_IQ24(0.01)
#define		cBus5V				_IQ24(0.005)
#define		cBus1V				_IQ24(0.001)

//Utility Voltage 
#define     c120VAC				1200
#define     c220VAC				2200
#define		c250VAC				2500
#define		c12VAC				120
#define		c22VAC				220

//Utility Freqency
#define		c60Hz				1
#define		c50Hz				2

//Frequency cal ratio
#define     cCntToFreq			187500000		// 60MHz * 100 / 32
#define     cFreqToQ24			6711			// 0.0001 * Q26
#define		c20HzCnt			93750			// 60MHz/20
#define		c100HzCnt			18750			// 60MHz/100

//Fault code
#define		cLineVoltOver		1
#define		cLineVoltUnder		2
#define		cLineFreqOver		3
#define		cLineFreqUnder		4
#define		cInvCurrOver		5
#define		cBusVoltOver		6
#define		cBusVoltUnder		7
#define		cShortCircuit		8
#define		cSoftStartFail		9

//LED,KEY period
#define		cSec1_2s				300
#define		cSec1_0s				250
#define		cSec0_6s				150	
#define		cSec0_5s				125
#define		cSec0_25s				62
#define		cSec0_1s				25

//Super time
#define cSuper20ms	5


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - LIN Communication
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#define	PktSizeLin				2

// Prototype statements for functions found within this file.
extern interrupt void Lina_Level0_ISR(void);
extern void PackTxBuffersLin(void);
extern void UnpackRxBuffersLin(void);
extern void SetupLin(void);
extern void error(void);

// Global variables
extern Uint16 sdataALin;     // Send data for SCI-A
extern Uint16 rdataALin;     // Received data for SCI-A


extern Uint32 LinL0IntVect;  //Vector for L0 interrupts

extern Uint16 iLin;			  //Generic iterator
Uint16 countLin = 0;          //test

//Uint16 Gui_wPanelVolt=0;
//Uint16 Gui_wPanelCurrent;
//Uint16 Gui_wBoostOutputVolt=0;
//Uint16 Gui_wPanelOutputPower;
//Uint16 Gui_wMPPT_Status;
//Uint16 Gui_wSysMode = 0x0004;



extern void (*RcvTaskPointerLin)(void); 			// State pointer for Command Packet Receive
extern Uint16	CmdPacketLin[PktSizeLin];

//void SendCmdByte(void);
extern void GetCmdEchoByteLin(void);
extern void SendIndexByteLin(void);
extern void GetIndexEchoByteLin(void);
extern void GetDataByteLin(void);
extern void EchoDataByteLin(void);
extern void SendDataByteLin(void);
extern void GetCompleteByteLin(void);
extern void ReadyForDataLin(void);


extern void Inquiry_Panel_Voltage(void);
extern void Inquiry_Panel_Current(void);
extern void Inquiry_Boost_Voltage(void);
extern void Inquiry_Panel_Output_Power(void);
extern void Inquiry_MPPT_Status(void);
extern void Send_Running_Mode(void);
extern void Send_LLC_Output_Voltage(void);
extern void Send_Turn_On_Command(void);
extern void Send_Turn_Off_Command(void);
extern void Send_Turn_On_MPPT_Command(void);
extern void Send_Turn_Off_MPPT_Command(void);
		


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// VARIABLE DECLARATIONS - CCS WatchWindow / GUI support
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// -------------------------------- FRAMEWORK --------------------------------------
void	sPowerOnMode(void);
void	sStandByMode(void);
void	sSoftStartMode(void);
void	sNormalInvMode(void);
void	sFaultMode(void);
void	sSwKeyDet(void);
void	sLedControl(void);
interrupt void ECap1_ISR(void);
extern  void 	InitECapture();
extern  void 	InitECap1Gpio(void);
int16   sRMSCal(long dwSum,long dwRatio,int wFormat);
int16	sAverageVoltCal(long dwSum,long dwRatio,int wFormat);
int16   sActivePowerCal(long dwSum,long dwRatio,int wFormat);

extern  void DPL_Zero_Buff(void);

#if 	(Protect_Enable == 1)
int16   sOverRangeChk(int16 wValue,int16 wHighLimit,int16 *wChkCnt,int16 wCntValue);
int16   sUnderRangeChk(int16 wValue,int16 wLowLimit,int16 *wChkCnt,int16 wCntValue);
int16   sShortCircuitChk(int16 wVolt,int16 wCurr,int16 *wChkCnt,int16 wCntValue);
int16   sRushChk(int16 wValue,int16 wRushLimit,int16 *wChkCnt,int16 wCntValue);
#endif

#ifdef FLASH
int16 *varGetList[]={&Gui_wInvVoltRms,&Gui_wInvFreq,&Gui_wDcBusVoltAvg,&Gui_wInvCurrRms,&Gui_wActivePower,
&Gui_wPanelVolt,&Gui_wBoostOutputVolt,&Gui_wPanelCurrent,&Gui_wPanelOutputPower,&Gui_wInvVolt_High,
&Gui_wInvVolt_Low,&Gui_wFreq_High,&Gui_wFreq_Low,&Gui_DC_Board_Connect,&Gui_wMPPT_Status,
&Gui_wSysMode,&Gui_wFaultCode,&wDataLogEnable,&Gui_wLogStatus};/*&Gui_wRLineVolt,&Gui_wSLineVolt,&Gui_wTLineVolt,&Gui_wBusVolt,
	&Gui_wRLineCurr,&Gui_wSLineCurr,&Gui_wTLineCurr,&Gui_wPowerFactor,
	&Gui_wTurnOnStatus,&Gui_wTurnOffStatus,&Gui_wLogStatus,
	(int16 *)&Gui_dwVolt_a1,(int16 *)&Gui_dwVolt_a2,(int16 *)&Gui_dwVolt_b1,(int16 *)&Gui_dwVolt_b2,
     (int16 *)&Gui_dwCurr_a1,(int16 *)&Gui_dwCurr_a2,(int16 *)&Gui_dwCurr_b0,(int16 *)&Gui_dwCurr_b1,(int16 *)&Gui_dwCurr_b2};
*/
int *varSetBtnList[]={&Gui_wTurnOn,&Gui_wTurnOff};
int16 *varSetSldrList[]={&Gui_wSampleLength,&Gui_wSampleRate};
int *varSetTxtList[]={&Gui_wSampleLength,&Gui_wSampleRate,
	&Gui_CH1_ID,&Gui_CH2_ID,&Gui_CH3_ID,&Gui_CH4_ID,&Gui_wLogEnable,&Gui_wFreq_High,
&Gui_wInvVolt_Low,&Gui_wFreq_High,&Gui_wFreq_Low,&Gui_wUtility_Setting,&Gui_GridTie_Enable,&Gui_wTurnOn,
&Gui_wTurnOff,&Gui_wInvVolt_High,&Gui_wSetting_Enable,&Gui_wLoadDefault_Enable,&Gui_wLogStatus};
	//&Gui_wSetting_V,&Gui_wSetting_I,&Gui_wUtilitySys};

long *varSetTxtLongList[]={&dwTestTemp};/*{&Gui_dwVolt_a1,&Gui_dwVolt_a2,&Gui_dwVolt_b1,&Gui_dwVolt_b2,
     &Gui_dwCurr_a1,&Gui_dwCurr_a2,&Gui_dwCurr_b0,&Gui_dwCurr_b1,&Gui_dwCurr_b2};
*/
int *arrayGetList[]={DBUFF1,DBUFF2,DBUFF3,DBUFF4};
#endif

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// MAIN CODE - starts here
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
void main(void)
{
    Uint16 buff[16], i, j, k, sum;        // for adc average
    float ft1;   // temporarary float value
//=================================================================================
//	INITIALISATION - General
//=================================================================================

	// The DeviceInit() configures the clocks and pin mux registers 
	// The function is declared in {ProjectName}-DevInit_F2803/2x.c,
	// Please ensure/edit that all the desired components pin muxes 
	// are configured properly that clocks for the peripherals used
	// are enabled, for example the individual PWM clock must be enabled 
	// along with the Time Base Clock 

	DeviceInit();	// Device Life support & GPIO
//	SCIA_Init();

//-------------------------------- FRAMEWORK --------------------------------------

// Only used if running from FLASH
// Note that the variable FLASH is defined by the compiler with -d FLASH
#ifdef FLASH		
// Copy time critical code and Flash setup code to RAM
// The  RamfuncsLoadStart, RamfuncsLoadEnd, and RamfuncsRunStart
// symbols are created by the linker. Refer to the linker files. 
	MemCopy(&RamfuncsLoadStart, &RamfuncsLoadEnd, &RamfuncsRunStart);

// Call Flash Initialization to setup flash wait states
// This function must reside in RAM
	InitFlash();	// Call the flash wrapper init function
#endif //(FLASH)

#ifdef FLASH
//	SCIA_Init();  	// Initalize the Serial Comms A peripheral - Uncomment for FLASH config
#endif //(FLASH)
// Timing sync for background loops
// Timer period definitions found in PeripheralHeaderIncludes.h
	CpuTimer0Regs.PRD.all =  mSec1;		// A tasks
	CpuTimer1Regs.PRD.all =  mSec5;		// B tasks
	CpuTimer2Regs.PRD.all =  mSec0_5;	// C tasks

// Tasks State-machine init
	Alpha_State_Ptr = &A0;
	A_Task_Ptr = &A1;
	B_Task_Ptr = &B1;
	C_Task_Ptr = &C1;

	BlinkStatePtr = 0;
	VTimer0[0] = 0;	
	VTimer1[0] = 0;
	VTimer2[0] = 0;
#ifdef	FLASH
	CommsOKflg = 0;
	SerialCommsTimer = 0;
#endif

// ---------------------------------- USER -----------------------------------------
//  put common initialization/variable definitions here


#if defined(DSP2802x_DEVICE_H) || defined(DSP2803x_DEVICE_H)

#endif

//===============================================================================
//	INITIALISATION - GUI connections
//=================================================================================
// Use this section only if you plan to "Instrument" your application using the 
// Microsoft C# freeware GUI Template provided by TI

#ifdef FLASH
	
#endif //(FLASH)

//Initialize, and calibrate ADC
InitAdc();
AdcOffsetSelfCal();
//initialize GPIO pins

//==================================================================================
//	INCREMENTAL BUILD OPTIONS - NOTE: selected via {ProjectName-Settings.h}
//==================================================================================
// ---------------------------------- USER -----------------------------------------
	// ADC Channel Selection for Configuring the ADC
	//Map channel to ADC Pin
	ChSel[0] =  0;
//	ChSel[1] =  1;		//
//	ChSel[2] =  2;      //
	ChSel[3] =  3;		//
	ChSel[4] =  4;      //
//	ChSel[5] =  5;		//
//	ChSel[6] =  6;		//
//	ChSel[7] =  7;		//
	ChSel[8] =  8;		//
	ChSel[9] =  9;		//
    ChSel[10] =  10;      //
    ChSel[11] =  11;      //
    ChSel[12] =  12;      //
//    ChSel[10] =  13;      //
//    ChSel[10] =  14;      //
//    ChSel[10] =  15;      //

			  
	// Select Trigger Event 
	TrigSel[0] = ADCTRIG_EPWM3_SOCA;	 
//	TrigSel[1] = ADCTRIG_EPWM3_SOCA;
//	TrigSel[2] = ADCTRIG_EPWM3_SOCA;
	TrigSel[3] = ADCTRIG_EPWM3_SOCA;		 
	TrigSel[4] = ADCTRIG_EPWM3_SOCA;		 
//	TrigSel[5] = ADCTRIG_EPWM3_SOCA;
//	TrigSel[6] = ADCTRIG_EPWM3_SOCA;
//	TrigSel[7] = ADCTRIG_EPWM3_SOCA;
	TrigSel[8] = ADCTRIG_EPWM3_SOCA;		 
	TrigSel[9] = ADCTRIG_EPWM3_SOCA;
    TrigSel[10] = ADCTRIG_EPWM3_SOCA;
    TrigSel[11] = ADCTRIG_EPWM3_SOCA;
    TrigSel[12] = ADCTRIG_EPWM3_SOCA;
//    TrigSel[13] = ADCTRIG_EPWM3_SOCA;
//    TrigSel[14] = ADCTRIG_EPWM3_SOCA;
//    TrigSel[15] = ADCTRIG_EPWM3_SOCA;


		 
#if TRANSPHORM_TDPVXXXXE0C1
	// Configure PWM1~3
	PWM_SolarInv_CNF(1, TRANSPHORM_PWM_PRD);
	PWM_SolarInv_CNF(2, TRANSPHORM_PWM_PRD);
	PWM_SolarInv_CNF(3, TRANSPHORM_LOOP_PRD);
#else
	// Configure PWM1~3
	PWM_SolarInv_CNF(1, 3125); // for 19.2Khz swicthing frequency
	PWM_SolarInv_CNF(2, 3125); // 
	PWM_SolarInv_CNF(3, 3125); //
#endif
	
	// Sync PWM1 with PWM2 without phase shift
	// Sync PWM1 with PWM3 with phase shift to trigger the ADC
	EPwm2Regs.TBCTL.bit.PHSEN = TB_ENABLE;
	EPwm2Regs.TBCTL.bit.PHSDIR = TB_UP;
	EPwm2Regs.TBPHS.half.TBPHS = 0;

#if 0	// Don't synch PWM3 to PWM1 since we are using it for loop decimation
	EPwm3Regs.TBCTL.bit.PHSEN = TB_ENABLE;
	EPwm3Regs.TBCTL.bit.PHSDIR = TB_UP;
	EPwm3Regs.TBPHS.half.TBPHS = 0;
#endif
	
	ADC_SOC_CNF(ChSel,TrigSel,ACQPS,12,0);           // was 4

	// Configure the Start of Conversion for the ADC. ePWM3 is the trigger.
	// At TBCNT1 = 0 start to convert all the channel
	EPwm3Regs.ETSEL.bit.SOCAEN = 1;
	EPwm3Regs.ETSEL.bit.SOCASEL = ET_CTR_ZERO;	// Use ZERO event as trigger
	//EPwm3Regs.ETSEL.bit.SOCASEL = ET_CTR_PRDZERO;	// Use ZERO & PERIOD event as trigger
    EPwm3Regs.ETPS.bit.SOCAPRD = ET_1ST;        // Generate pulse on 1st event  
    
    InitECapture();      
 	
	// Digital Power (DP) library initialisation 
	DPL_Init();
	
	ADCDRV_5ch_RltPtrA = &dwInv_Current_1; 	// instance #0
	ADCDRV_5ch_RltPtrB = &dwInv_Voltage; 	// instance #1
	ADCDRV_5ch_RltPtrC = &dwBus_Voltage_Fbk; 	// instance #2dwLine_Voltage
	ADCDRV_5ch_RltPtrD = &dwMid_Ref_Volt; 	// instance #3
	ADCDRV_5ch_RltPtrE = &dwPLCResr; 	// instance #4
	
	//2P2Z net
	//Voltage loop 
	CNTL_2P2Z_Ref1 = &dwBus_Voltage_Ref;	
	CNTL_2P2Z_Out1 = &dwBus_Voltage_Loop_Out_1;
	CNTL_2P2Z_Fdbk1 = &dwBus_Voltage_Fbk;	
	CNTL_2P2Z_Coef1 = &CNTL_2P2Z_CoefStruct1.b2;

	//Current loop
	CNTL_2P2Z_Ref2 = &dwInv_Curr_Ref;
	CNTL_2P2Z_Out2 = &dwInv_Curr_LoopOut;
	CNTL_2P2Z_Fdbk2 = &dwInv_Current_1;	 
	CNTL_2P2Z_Coef2 = &CNTL_2P2Z_CoefStruct2.b2; 	 
	
	// Notch filter
 	CNTL_2P2Z_Ref3 = &dwPhase_Error_Out;	 
	CNTL_2P2Z_Out3 = &dwPhase_Err_Notch_Out;	 
	CNTL_2P2Z_Fdbk3 = &dwZero; 
	CNTL_2P2Z_Coef3 = &CNTL_2P2Z_CoefStruct3.b2;  

	// CONTROL_2P2Z
	CNTL_2P2Z_Ref4 = &dwPhase_Err_Notch_Out;	 
	CNTL_2P2Z_Out4 = &dwPllCon_Out; 
	CNTL_2P2Z_Fdbk4 = &dwZero;	 
	CNTL_2P2Z_Coef4 = &CNTL_2P2Z_CoefStruct4.b2; 	 
	 	
 	INV_ICMD_Vcmd1 = &dwBus_Voltage_Loop_Out;
    //INV_ICMD_Vac1 = &dwLine_Voltage;
    INV_ICMD_Vac1 = &dwSine_Ref; 		 
	INV_ICMD_Out1 = &dwInv_Curr_Ref;		 
	INV_ICMD_Comp1 = &dwFdFwd_Comp;
 	INV_ICMD_Out_Max1 = &dwInvCurrRef_Max;
	INV_ICMD_Out_Min1 = &dwInvCurrRef_Min;
	
	PWMDRV_Duty = &dwDuty_Cal_Out;
	//PWMDRV_Duty =&dwInv_Voltage;		//Open loop test
 	PWMDRV_Temp = &dwDuty_Temp;       
 	PWMDRV_Ratio = &dwPWM_Cal_Ratio;   
 	
 	GEN_SIN_COS_Sin_0 = &dwPll_Sin_0;			 
	GEN_SIN_COS_Sin_1 = &dwPll_Sin_Out;			 
	GEN_SIN_COS_Cos_0 = &dwPll_Cos_0;		 
	GEN_SIN_COS_Cos_1 = &dwPll_Cos_out;			 
	GEN_SIN_COS_Max = &dwPll_Sin_Cos_Max;  	 
	GEN_SIN_COS_Min = &dwPll_Sin_Cos_Min;  	 
	GEN_SIN_COS_Ws = &dwPll_Trace_Freq;		 
	GEN_SIN_COS_Ts = &dwPll_Sample_Time;	
	
	DLOG_4ch_i1Ptr =&dwInv_Current_1;
	DLOG_4ch_i2Ptr =&dwInv_Curr_Ref;
	DLOG_4ch_i3Ptr =&dwInv_Voltage;
	DLOG_4ch_i4Ptr =&dwBus_Voltage_Fbk;
    	
	DLOG_4ch_buff1Ptr =DBUFF1;
	DLOG_4ch_buff2Ptr =DBUFF2;
	DLOG_4ch_buff3Ptr =DBUFF3;
	DLOG_4ch_buff4Ptr =DBUFF4;
	
	DLOG_4ch_TrigVal = _IQ24(0.1);
	DLOG_4ch_PreScalar = 4;	
	DLOG_4ch_Size=DLOG_SIZE; 
	
	dwPWM_Cal_Ratio = _IQ8(3905);      //1562*1000/Vbus
	
	dwBus_Voltage_Ref = _IQ24(0.0);
	dwBus_Voltage_Loop_Out = _IQ24(0.0);
	dwBus_Voltage_Loop_Out_1 = _IQ24(0.0);
	dwInv_Curr_LoopOut = _IQ24(0.0);
	dwInv_Curr_Ref = _IQ24(0.0);
	
	//Voltage loop
	CNTL_2P2Z_CoefStruct1.b2 = _IQ26(-0.3682);
	CNTL_2P2Z_CoefStruct1.b1 = _IQ26(0.01121);
	CNTL_2P2Z_CoefStruct1.b0 = _IQ26(0.3794);
	CNTL_2P2Z_CoefStruct1.a2= _IQ26(-0.869);
	CNTL_2P2Z_CoefStruct1.a1= _IQ26(1.869);
	CNTL_2P2Z_CoefStruct1.max = _IQ24(0.0);		//4A
	CNTL_2P2Z_CoefStruct1.min = _IQ24(-0.4);

	//Current loop for 52e-6 sample time
	CNTL_2P2Z_CoefStruct2.b2 = _IQ26(-0.3938);
	CNTL_2P2Z_CoefStruct2.b1 = _IQ26(0.1314);
	CNTL_2P2Z_CoefStruct2.b0 = _IQ26(0.5253);
	CNTL_2P2Z_CoefStruct2.a2= _IQ26(-0.01);
	CNTL_2P2Z_CoefStruct2.a1= _IQ26(1.01);
	CNTL_2P2Z_CoefStruct2.max = _IQ24(0.4);
	CNTL_2P2Z_CoefStruct2.min = _IQ24(-0.4);		//400
	
	//Notch filter for 60Hz system, sample rate 416e-6s
	CNTL_2P2Z_CoefStruct3.b2 = _IQ26(0.9);
	CNTL_2P2Z_CoefStruct3.b1 = _IQ26(-1.712);
	CNTL_2P2Z_CoefStruct3.b0 = _IQ26(0.9);
	CNTL_2P2Z_CoefStruct3.a2= _IQ26(-0.8);
	CNTL_2P2Z_CoefStruct3.a1= _IQ26(1.712);
	CNTL_2P2Z_CoefStruct3.max = _IQ24(0.99);		// rad
	CNTL_2P2Z_CoefStruct3.min = _IQ24(-0.99);		// rad
	
	//PLL controller , sample rate 416e-6
	CNTL_2P2Z_CoefStruct4.b2 = _IQ26(-0.01723);
	CNTL_2P2Z_CoefStruct4.b1 = _IQ26(0.0004355);
	CNTL_2P2Z_CoefStruct4.b0 = _IQ26(0.01767);
	CNTL_2P2Z_CoefStruct4.a2= _IQ26(-0.864);
	CNTL_2P2Z_CoefStruct4.a1= _IQ26(1.864);
	CNTL_2P2Z_CoefStruct4.max = _IQ24(0.5);		// 50Hz
	CNTL_2P2Z_CoefStruct4.min = _IQ24(-0.5);		// -50Hz
	
	//Generate Sin and Cos
	dwPll_Sin_0 = _IQ22(0.0);			 
	dwPll_Sin_Out = _IQ22(0.0);			 
	dwPll_Cos_0 = _IQ22(0.99);		 
	dwPll_Cos_out = _IQ22(0.99);			 
	dwPll_Sin_Cos_Max = _IQ22(0.99);  	 
	dwPll_Sin_Cos_Min = _IQ22(-0.99);

	/* Uncomment one value below for dwPll_Trace_Freq depending on desired grid frequency */
	dwPll_Trace_Freq = _IQ20(376.9911);	 				//60Hz
	//dwPll_Trace_Freq = _IQ20(314.1593);	 				//50Hz

#if TRANSPHORM_TDPVXXXXE0C1
	dwPll_Sample_Time = _IQ24(TRANSPHORM_SAMPLE_TIME);
#else
	dwPll_Sample_Time = _IQ24(0.00005207);				//52e-6
#endif
	
	dwRef_Sin_Cal_Ratio = _IQ20(5.893);			//3020*3.3/10*Vrms*1.414
	dwRef_Sin_Out = _IQ22(0.0);
	dwPhase_Error_Out = _IQ24(0.0);
    dwPhase_Err_Max = _IQ24(1.0);					
    dwPhase_Err_Min = _IQ24(-1.0);
    dwPhase_Err_Offset = _IQ24(0.06);
	
	dwDuty_Cal_Max = _IQ24(0.4);				//400v
	dwDuty_Cal_Min = _IQ24(-0.4);
	
	dwInv_Frequency = _IQ24(0.6);  				//60Hz
	
	dwFdFwd_Comp = _IQ26(1.0);		//3020*3.3/(4.99*Vrms*1.414)
	dwInvCurrRef_Max = _IQ24(0.4357);		//9A * 0.8 * 4.99/(24.99*3.3)
	dwInvCurrRef_Min = _IQ24(-0.4357);
    
    wSysMode = cPowerOnMode;
	FSuperFlag.word = 0;
	VTimer0[1] = 0;
	wCycleFlag = 0;
 	wTestTempCnt = 0;
#if TRANSPHORM_TDPVXXXXE0C1
	wSinAmp = _IQ10(TRANSPHORM_SIN_AMP);
#else
 	wSinAmp = _IQ10(0.311);
#endif
	wSinAmp_I = _IQ10(0.2);
	wBus_V = _IQ10(0.408);
 	wDataLogEnable = 0;
 	wStartCheckCnt = 0;
 	wSysStatusCheckCnt = 0;
 	wGPINCheckCnt = 0;
	
	mModeLed_OFF();
	mFaultLed_OFF();
	
	//Calculation variables
	dwInvVoltSumTemp = 0;
	dwInvVoltSum = 0;
	dwInvCurrSumTemp = 0;
	dwInvCurrSum = 0;
	dwDcBusSumTemp = 0;
	dwDcBusSum = 0;
	dwActivePowerSumTemp = 0;
	dwActivePowerSum = 0;
	wSumCnt = 0;
	wSumCntTemp = 0;
	wLineCycleFlag = 0;
	wNoZeroCrossCnt = 0;
	wCalculationCnt = 0;
	wTurnOffByButton = 0;
	wGridTieEnable = 0;
	wNoSciRevIntCnt = 0;
	
	dwInv_Current_Ratio = _IQ18(20.6612);
	dwInv_Voltage_Ratio= _IQ18(996.6);
	dwBus_Voltage_Ratio = _IQ18(993.3);
	dwActivePower_Ratio = _IQ10(20590.95);	
	
	dwVoltageLoopOutConst = _IQ24(0.18);
	
	//GUI interface variables
	Gui_wInvVoltRms = 0;
	Gui_wInvCurrRms = 0;
	Gui_wDcBusVoltAvg = 0;
	Gui_wActivePower = 0;
	Gui_wInvFreq = 0;
	Gui_wFaultCode = 0;
	Gui_Bus_Volt_Ref = cBus380V;
	
	Gui_wPanelVolt = 2280;
	Gui_wPanelCurrent = 170; 
	Gui_wBoostOutputVolt = 3988;
	Gui_wPanelOutputPower = 366;
	
	Gui_wInvVoltOverChkCnt = 0;
	Gui_wInvVoltUnderChkCnt = 0;	
	Gui_wInvCurrChkCnt = 0;
	Gui_wInvCurr_High_Limit = 7000;		//7A
	Gui_wBusVoltOverChkCnt = 0;
	Gui_wBusVoltUnderChkCnt = 0;
	Gui_wBusVolt_High_Limit = 4100;		//430V
	Gui_wBusVolt_Low_Limit = 3200;		//320V
	Gui_wBusVoltRushChkCnt = 0;
	Gui_wBusVolt_Rush_Limit = 4500;		//450V
	Gui_wFreqOverChkCnt = 0;
	Gui_wFreqUnderChkCnt = 0;
	Gui_wShortCircuitChkCnt = 0;
	
	Gui_wLoadDefault_Enable = 0;
	Gui_wSetting_Enable = 0;
	Gui_Utility_Setting_Enable = 1;
	Gui_wUtility_Setting = 1;
	if(Gui_wUtility_Setting == 1)
	{
		Gui_Utility_Freq = c60Hz;
		Gui_Utility_Volt = c120VAC;
		wCyclePeriodCnt = 320;
		Gui_wInvVolt_High_Limit = (Gui_Utility_Volt + c12VAC);		
		Gui_wInvVolt_Low_Limit = (Gui_Utility_Volt - c12VAC);	
		Gui_wInvVolt_High = Gui_wInvVolt_High_Limit;
		Gui_wInvVolt_Low = Gui_wInvVolt_Low_Limit;
		Gui_wFreq_High_Limit = 6600;		//55Hz
		Gui_wFreq_Low_Limit = 5400;			//45Hz	
		Gui_wFreq_High = Gui_wFreq_High_Limit;
		Gui_wFreq_Low = Gui_wFreq_Low_Limit;
		
		//Notch filter for 60Hz system, sample rate 416e-6s
		CNTL_2P2Z_CoefStruct3.b2 = _IQ26(0.9);
		CNTL_2P2Z_CoefStruct3.b1 = _IQ26(-1.712);
		CNTL_2P2Z_CoefStruct3.b0 = _IQ26(0.9);
		CNTL_2P2Z_CoefStruct3.a2= _IQ26(-0.8);
		CNTL_2P2Z_CoefStruct3.a1= _IQ26(1.712);
		CNTL_2P2Z_CoefStruct3.max = _IQ24(0.99);		// rad
		CNTL_2P2Z_CoefStruct3.min = _IQ24(-0.99);		// rad
	}
	else if(Gui_wUtility_Setting == 2)
	{
		Gui_Utility_Freq = c50Hz;
		Gui_Utility_Volt = c220VAC;
		wCyclePeriodCnt = 384;
		Gui_wInvVolt_High_Limit = (Gui_Utility_Volt + c12VAC);		
		Gui_wInvVolt_Low_Limit = (Gui_Utility_Volt - c12VAC);		
		Gui_wInvVolt_High = Gui_wInvVolt_High_Limit;
		Gui_wInvVolt_Low = Gui_wInvVolt_Low_Limit;
		Gui_wFreq_High_Limit = 5500;		//55Hz
		Gui_wFreq_Low_Limit = 4500;			//45Hz	
		Gui_wFreq_High = Gui_wFreq_High_Limit;
		Gui_wFreq_Low = Gui_wFreq_Low_Limit;
		
		//Notch filter for 50Hz system, sample rate 416e-6s
		CNTL_2P2Z_CoefStruct3.b2 = _IQ26(0.9);
		CNTL_2P2Z_CoefStruct3.b1 = _IQ26(-1.739);
		CNTL_2P2Z_CoefStruct3.b0 = _IQ26(0.9);
		CNTL_2P2Z_CoefStruct3.a2= _IQ26(-0.8);
		CNTL_2P2Z_CoefStruct3.a1= _IQ26(1.739);
		CNTL_2P2Z_CoefStruct3.max = _IQ24(0.99);		// rad
		CNTL_2P2Z_CoefStruct3.min = _IQ24(-0.99);		// rad
	}
	
	Gui_wTurnOn = 0;
	Gui_wTurnOff = 0;
	
	Gui_wGridTie_Status = 0;
	Gui_wMPPT_Status = 0;
	Gui_DC_Board_Connect = 0;
	
	Gui_wSysMode = 0;
	Gui_GridTie_Enable = 0;	//Disable the grid-tie
	
	Gui_wSampleRate = 4;
    Gui_wSampleLength = DLOG_SIZE;
	Gui_CH1_ID = 1;
	Gui_CH2_ID = 2;
	Gui_CH3_ID = 3;
	Gui_CH4_ID = 4;
	Gui_wLogEnable = 0;
	Gui_wLogStatus = 0;
	
	FSuperFlag.BIT.FwAutoStartOnEn = 0;		//Disable the auto start

//==============================================================================
// Items common to all builds
//==============================================================================
	mFREE_RUN_OFF();					
	mOPRLY_OFF();	
	mALL_PWM_OFF();


//All enabled ePWM module clocks are started with the first rising edge of 
//TBCLK aligned   
	EALLOW;
	SysCtrlRegs.PCLKCR0.bit.TBCLKSYNC = 1;
	EDIS;
	
	
//==============================================================================
// LIN Initialize
//==============================================================================
	//Initialize Variables
   	LinL0IntVect  = 0;
   	
   	//EALLOW;
    //InitLinGpio();
   	//DINT;
   	EALLOW;
	GpioCtrlRegs.GPAPUD.bit.GPIO9 = 0;		// Enable pull-up for GPIO9 (LIN TX)
	GpioCtrlRegs.GPAPUD.bit.GPIO11 = 0;		// Enable pull-up for GPIO11 (LIN RX)
	GpioCtrlRegs.GPAQSEL1.bit.GPIO11 = 3;  // Asynch input GPIO11 (LINRXA)

	/* Configure LIN-A pins using GPIO regs*/
	GpioCtrlRegs.GPAMUX1.bit.GPIO9 = 2;    // Configure GPIO9 for LIN TX operation (2-Enable,0-Disable)
	GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 2;   // Configure GPIO11 for LIN RX operation	 (2-Enable,0-Disable)
    EDIS;
   	
   	//Initialize and Enable BLIN SCI module
    SetupLin();
    
    sdataALin = 0x0;
   	rdataALin = 0x0;
   	
   	RcvTaskPointerLin = &GetCmdEchoByteLin;			// Initialize the CmdPacket Rcv Handler state machine ptr
   	
   	// clear Command Packet
	for (iLin=0; iLin<PktSizeLin; iLin++)
	{
		CmdPacketLin[iLin] = 0x0;
	}


//=================================================================================
//	INTERRUPT & ISR INITIALISATION (best to run this section after other initialisation)
//=================================================================================
//Also Set the appropriate # define's in the {ProjectName}-Settings.h 
//to enable interrupt management in the ISR

	EALLOW;
	PieVectTable.ADCINT1 = &DPL_ISR;
	PieVectTable.ECAP1_INT = &ECap1_ISR;
	PieVectTable.LIN0INTA  = &Lina_Level0_ISR;
	EDIS;
	
	EALLOW;
// Enable eCAP INTn in the PIE: Group 3 interrupt 1-6
   PieCtrlRegs.PIEIER1.bit.INTx1 = 1;	// Enable INT 1.1 in the PIE
   //PieCtrlRegs.PIEIER4.bit.INTx1 = 1;
   
   //LIN Enable interrupts required for this example
	//PieCtrlRegs.PIECTRL.bit.ENPIE = 1;   // Enable the PIE block
	PieCtrlRegs.PIEIER9.bit.INTx3 = 1;     // PIE Group 9, INT3
	IER = 0x100;	// Enable CPU INT
	
// Enable Peripheral, global Ints and higher priority real-time debug events:
   IER |= M_INT1;

// Enable CPU INT4 which is connected to ECAP1-4 INT:
   IER |= M_INT4;
   	EDIS;

	EINT;   // Enable Global interrupt INTM
	ERTM;   // Enable Global realtime interrupt DBGM 

//=================================================================================
//	BACKGROUND (BG) LOOP
//=================================================================================

	//Wait for SCI to be idle and ready for transmission
//	while(LinaRegs.SCIFLR.bit.IDLE == 1);
	Gui_wPanelVolt = 0;
	Gui_wPanelCurrent = 0; 
	Gui_wBoostOutputVolt = 0;
	Gui_wPanelOutputPower = 0;
	
	wSendTurnOnCmd = 0;
    wSendTurnOffCmd = 0;

#if TRANSPHORM_TDPVXXXXE0C1
    FSuperFlag.BIT.FwTurnOn = 1;	// Force Turn-On
#endif

    mFaultLed_OFF();
    j=0;  sum=0;
    for(i=0; i<500; i++){                      // spend a lot of time here to let things settle...get last 16 values...
      for(j=0; j<=500; j++){
        buff[k] =  AdcResult.ADCRESULT4;
      }
      if(k++ >= 15) k=0;  // reset if needed;
    }

    for(i=0; i<=15; i++){
        sum += buff[i];
    }

    sum >>= 4;

    Pvars.ACzero = sum;             // get the average true zero

    ft1 = (float)Pvars.ACzero;
    ft1 /= 2048;
    ft1 *= 512;
    Pvars.DACzero = (Uint16)(ft1);      // adjust the DAC for true zero

    Pvars.FaultLock = 1;
    Pvars.Pstate = Start;
    Pvars.AmpsTimer = 25;
    Pvars.DelayTimer = DELAY_COUNTER;
    Pvars.AmpOut = 0;
    Pvars.PWMonSyncFlag = 0;

    Pvars.OverCurrentTrip = 4096;   // load with a value that can't be reached to disable for now

    Pvars.OverVoltageTrip = (Uint16)(OVER_VOLTAGE_TRIP * VOLTAGE_SCALING);          // load scaled over voltage setting
    Pvars.UnderVoltageTrip = (Uint16)(UNDER_VOLTAGE_TRIP * VOLTAGE_SCALING);        // load scaled under voltage setting
    Pvars.Hysteresis = (Uint16)(HYSTERESIS_SET * VOLTAGE_SCALING);                  // load scaled hysteresis for under voltage
    Pvars.DAChighSide = (Uint16)(COMPARATOR_MAX * CURRENT_SCALING_DAC);             // load scaled positive going current trip point
    Pvars.DAClowSide = (Uint16)(COMPARATOR_MIN * CURRENT_SCALING_DAC);              // load scaled negative going current trip point
    Pvars.UnderVoltEnable = UNDER_VOLTAGE_ENABLE;                                   // load enable bit for undervoltage feature

    TripZoneInit();                 // set up trip zones for current protection

//--------------------------------- FRAMEWORK -------------------------------------
	for(;;)  //infinite loop
	{
		// State machine entry & exit point
		//===========================================================
		(*Alpha_State_Ptr)();	// jump to an Alpha state (A0,B0,...)
		//===========================================================
		//SerialHostComms();
		LoopCounter++;
		ProtectionPolling();
		if(PolarityBit != Pvars.SinePolarity){        // go in here when we change polarities
		    if(Pvars.PWMonSyncFlag) {
		        Pvars.PWMonSyncFlag = 0;        // set done flag
		        mALL_PWM_ON();                  // enable pwm
		    }
		    MoveCurrentLimtBar();   // this function moves the current limit bar when the polarity changes state
		    Pvars.SinePolarity = PolarityBit;    // update the polarity bit
		}
//		if(Comp2Regs.COMPSTS.bit.COMPSTS) GpioDataRegs.GPASET.bit.GPIO28 = 1;   // for de-bug
//		else GpioDataRegs.GPACLEAR.bit.GPIO28 = 1;                              // for de-bug
	}
}    //END MAIN CODE



//=================================================================================
//	STATE-MACHINE SEQUENCING AND SYNCRONIZATION
//=================================================================================

//--------------------------------- FRAMEWORK -------------------------------------
void A0(void)
{
	// loop rate synchronizer for A-tasks
	if(CpuTimer0Regs.TCR.bit.TIF == 1)
	{
		CpuTimer0Regs.TCR.bit.TIF = 1;	// clear flag

		//-----------------------------------------------------------
		(*A_Task_Ptr)();		// jump to an A Task (A1,A2,A3,...)
		//-----------------------------------------------------------

		VTimer0[0]++;			// virtual timer 0, instance 0 (spare)
		SerialCommsTimer++;
	}

	Alpha_State_Ptr = &B0;		// Comment out to allow only A tasks
}

void B0(void)
{
	// loop rate synchronizer for B-tasks
	if(CpuTimer1Regs.TCR.bit.TIF == 1)
	{
		CpuTimer1Regs.TCR.bit.TIF = 1;				// clear flag

		Pvars.LoopRate = LoopCounter;
		LoopCounter = 0;

		//-----------------------------------------------------------
		(*B_Task_Ptr)();		// jump to a B Task (B1,B2,B3,...)
		//-----------------------------------------------------------
		VTimer1[0]++;			// virtual timer 1, instance 0 (spare)
	}

	Alpha_State_Ptr = &C0;		// Allow C state tasks
	
	//Change the gain for each close loop
	
}

void C0(void)
{
	// loop rate synchronizer for C-tasks
	if(CpuTimer2Regs.TCR.bit.TIF == 1)
	{
		CpuTimer2Regs.TCR.bit.TIF = 1;				// clear flag

		//-----------------------------------------------------------
		(*C_Task_Ptr)();		// jump to a C Task (C1,C2,C3,...)
		//-----------------------------------------------------------
		VTimer2[0]++;			//virtual timer 2, instance 0 (spare)
		
		#ifdef FLASH
//		SerialHostComms();		//SCIA
		#endif
	}

	Alpha_State_Ptr = &A0;		// Back to State A0
}


//=================================================================================
//	A - TASKS
//=================================================================================
//--------------------------------------------------------
void A1(void) // System mode supervisor
//--------------------------------------------------------
{
	//the next time CpuTimer0 'counter' reaches Period value go to A2 or A1
	A_Task_Ptr = &A2;
	//-------------------
	VTimer0[1]++;
	if(VTimer0[1] >= cSuper20ms)	//20ms
	{
		VTimer0[1] = 0;		//Roll over
		
		switch(wSysMode)
		{
			case cPowerOnMode:
			sPowerOnMode();
			break;
			
			case cStandByMode:
			sStandByMode();
			break;
			
			case cSoftStartMode:
			sSoftStartMode();
			break;
			
			case cNormalInvMode:
			sNormalInvMode();
			break;
			
			case cFaultMode:
			sFaultMode();
			break;
			
			default:
			break;
		}
	
	}
}
//-----------------------------------------------------------------
void A2(void) // SCI GUI
//-----------------------------------------------------------------
{	
		//	EPwm1Regs.ETCLR.bit.INT=0x1;
	//	PieCtrlRegs.PIEACK.bit.ACK3=0x1; 
	//SerialHostComms();	// Uncomment for FLASH config
						// Serialport controls LED2 (GPIO-31)
						// Will not blink until GUI is connected 
	//-------------------
	//the next time CpuTimer0 'counter' reaches Period value go to A1
	A_Task_Ptr = &A3;	// To make task A3 active, change &A1 to &A3
	
	//-------------------
}
//-----------------------------------------
void A3(void)	//LED,Key control
//-----------------------------------------
{
	
	//-----------------
	//the next time CpuTimer0 'counter' reaches Period value go to A1
	A_Task_Ptr = &A4;
	//-----------------
	
	sSwKeyDet();	 
	sLedControl();
}


//----------------------------------------------------------
void A4(void) 
//---------------------------------------------------------
{	
	//-----------------
	//the next time CpuTimer0 'counter' reaches Period value go to A1
	A_Task_Ptr = &A1;
	//-----------------
}

//=================================================================================
//	B - TASKS
//=================================================================================

//----------------------------------- USER ----------------------------------------

//----------------------------------------
void B1(void) 
//----------------------------------------
{	
	B_Task_Ptr = &B2;	
	
	#if(Protect_Enable == 1)

	if(wCalculationCnt >= 5)
	{
		//Short circuit check
		if(sShortCircuitChk(Gui_wInvVoltRms,Gui_wInvCurrRms,&Gui_wShortCircuitChkCnt,2))
		{
			FSuperFlag.BIT.FwFault = 1;
			FSuperFlag.BIT.FwTurnOnConditionOk = 0;
			Gui_wFaultCode = cShortCircuit;
			wSysStatusCheckCnt = 0;
			return;
		}
		
		//Over Current
		if(sOverRangeChk(Gui_wInvCurrRms,Gui_wInvCurr_High_Limit,&Gui_wInvCurrChkCnt,6) == 1)
		{
			FSuperFlag.BIT.FwFault = 1;
			Gui_wFaultCode = cInvCurrOver;
			FSuperFlag.BIT.FwTurnOnConditionOk = 0;
			wSysStatusCheckCnt = 0;
			if((FSuperFlag.BIT.FwAutoStartOnEn == 1) && (wSysMode == cNormalInvMode))
			{
				FSuperFlag.BIT.FwAutoStart = 1;
			}
			return;
		}
		
		#if(Protect_Utiligy_Enable == 1)
		if(wGridTieEnable == 1)
		{
			//Line Overvoltage Check
			if(sOverRangeChk(Gui_wInvVoltRms,Gui_wInvVolt_High_Limit,&Gui_wInvVoltOverChkCnt,6) == 1)
			{
				if((wSysMode == cNormalInvMode) || (wSysMode == cSoftStartMode))
				{
					FSuperFlag.BIT.FwFault = 1;
					Gui_wFaultCode = cLineVoltOver;
				}
				else if(wSysMode == cStandByMode)
				{
					FSuperFlag.BIT.FwWarning = 1;
				}	
				FSuperFlag.BIT.FwTurnOnConditionOk = 0;
				wSysStatusCheckCnt = 0;
				if((FSuperFlag.BIT.FwAutoStartOnEn == 1) && (wSysMode == cNormalInvMode))
				{
					FSuperFlag.BIT.FwAutoStart = 1;
				}
				return;
			}
				
			//Line Under voltage check
			if(sUnderRangeChk(Gui_wInvVoltRms,Gui_wInvVolt_Low_Limit,&Gui_wInvVoltUnderChkCnt,6) == 1)
			{
				if((wSysMode == cNormalInvMode) || (wSysMode == cSoftStartMode))
				{
					FSuperFlag.BIT.FwFault = 1;
					Gui_wFaultCode = cLineVoltUnder;
				}
				else if(wSysMode == cStandByMode)
				{
					FSuperFlag.BIT.FwWarning = 1;
				}	
				FSuperFlag.BIT.FwTurnOnConditionOk = 0;
				wSysStatusCheckCnt = 0;
				if((FSuperFlag.BIT.FwAutoStartOnEn == 1) && (wSysMode == cNormalInvMode))
				{
					FSuperFlag.BIT.FwAutoStart = 1;
				}
				return;
			}
			
			//Frequency over check
			if(sOverRangeChk(Gui_wInvFreq,Gui_wFreq_High_Limit,&Gui_wFreqOverChkCnt,6) == 1)
			{
				if((wSysMode == cNormalInvMode) || (wSysMode == cSoftStartMode))
				{
					FSuperFlag.BIT.FwFault = 1;
					Gui_wFaultCode = cLineFreqOver;
				}
				else if(wSysMode == cStandByMode)
				{
					FSuperFlag.BIT.FwWarning = 1;
				}	
				FSuperFlag.BIT.FwTurnOnConditionOk = 0;
				wSysStatusCheckCnt = 0;
				if((FSuperFlag.BIT.FwAutoStartOnEn == 1) && (wSysMode == cNormalInvMode))
				{
					FSuperFlag.BIT.FwAutoStart = 1;
				}
				return;
			}
			//Freqency under check
			if(sUnderRangeChk(Gui_wInvFreq,Gui_wFreq_Low_Limit,&Gui_wFreqUnderChkCnt,6) == 1)
			{
				if((wSysMode == cNormalInvMode) || (wSysMode == cSoftStartMode))
				{
					FSuperFlag.BIT.FwFault = 1;
					Gui_wFaultCode = cLineFreqUnder;
				}
				else if(wSysMode == cStandByMode)
				{
					FSuperFlag.BIT.FwWarning = 1;
				}	
				FSuperFlag.BIT.FwTurnOnConditionOk = 0;
				wSysStatusCheckCnt = 0;
				if((FSuperFlag.BIT.FwAutoStartOnEn == 1) && (wSysMode == cNormalInvMode))
				{
					FSuperFlag.BIT.FwAutoStart = 1;
				}
				return;
			}
		}		
		#endif	

		//Bus Over Check
		if(sOverRangeChk(Gui_wDcBusVoltAvg,Gui_wBusVolt_High_Limit,&Gui_wBusVoltOverChkCnt,1) == 1)
		{
			FSuperFlag.BIT.FwFault = 1;
			FSuperFlag.BIT.FwTurnOnConditionOk = 0;
			Gui_wFaultCode = cBusVoltOver;
			wSysStatusCheckCnt = 0;
			if((FSuperFlag.BIT.FwAutoStartOnEn == 1) && (wSysMode == cNormalInvMode))
			{
				//FSuperFlag.BIT.FwAutoStart = 1;
			}
			return;
		}
		
			
		if((wSysMode == cNormalInvMode) || (wSysMode == cSoftStartMode))
		{
			//Bus Under Check
			if(sUnderRangeChk(Gui_wDcBusVoltAvg,Gui_wBusVolt_Low_Limit,&Gui_wBusVoltUnderChkCnt,6) == 1)
			{
				FSuperFlag.BIT.FwFault = 1;
				FSuperFlag.BIT.FwTurnOnConditionOk = 0;
				Gui_wFaultCode = cBusVoltUnder;
				wSysStatusCheckCnt = 0;
				Gui_wBusVoltUnderChkCnt = 0;
				if(FSuperFlag.BIT.FwAutoStartOnEn == 1)
				{
					//FSuperFlag.BIT.FwAutoStart = 1;
				}
				return;
			}
		}
		
		if(FSuperFlag.BIT.FwFault == 0)
		{
			wSysStatusCheckCnt++;
			if(wSysStatusCheckCnt >= 50)	//1s check
			{
				FSuperFlag.BIT.FwTurnOnConditionOk = 1;		
				wSysStatusCheckCnt = 50;
			}	
		}
		else
		{
			if(FSuperFlag.BIT.FwAutoStart == 1)
			{
				wSysStatusCheckCnt++;
				if(wSysStatusCheckCnt >= 50)	//1s check
				{
					FSuperFlag.BIT.FwClearFaultFlag = 1;	
					FSuperFlag.BIT.FwAutoStart = 0;	
					wSysStatusCheckCnt = 0;
				}
				
			}
		}
		
	}	
	#endif
	
	
}

//----------------------------------------
void B2(void)
//----------------------------------------
{
	//the next time CpuTimer1 'counter' reaches Period value go to B3
	B_Task_Ptr = &B3;
	//-----------------
	
	//Get the system status
	Gui_wSysMode = wSysMode;
	
	//Calculate the V,I,P
	if(wCycleFlag == 1)
	{
		wNoZeroCrossCnt = 0;
		
		wLineCycleFlag = 0;
		wCycleFlag = 0;
		wCalculationCnt++;
		
		if(wCalculationCnt >= 5)
		{
			wCalculationCnt = 5;
		}
		
		//Inv Voltage rms cal
		Gui_wInvVoltRms = sRMSCal(dwInvVoltSum,dwInv_Voltage_Ratio,10);
		
		//Inv Current rms cal
		Gui_wInvCurrRms = sRMSCal(dwInvCurrSum,dwInv_Current_Ratio,100);
		
		//DC Bus Voltage 
		Gui_wDcBusVoltAvg = sAverageVoltCal(dwDcBusSum,dwBus_Voltage_Ratio,10);
		
		DINT;
		Gui_wLLCOutputVolt = Gui_wDcBusVoltAvg;
		EINT;
		
		//Active Power cal
		Gui_wActivePower = sActivePowerCal(dwActivePowerSum,dwActivePower_Ratio,1);
	}
	else
	{
		wNoZeroCrossCnt++;
		if(wNoZeroCrossCnt >= 4)		//no Zero crossing in 40ms cycles
		{
			Gui_wInvFreq = 0;
			wNoZeroCrossCnt = 4;
		}
	}
}

//----------------------------------------
void B3(void)  //  Send command
//----------------------------------------
{
	//the next time CpuTimer1 'counter' reaches Period value go to B1
	B_Task_Ptr = &B4;	
	//-----------------
	
	if(Gui_DC_Board_Connect == 0)	//un-connected
	{		
		if(mGPIN_STATUS() == 0)
		{
			wGPINCheckCnt++;
			if(wGPINCheckCnt >= 5)
			{
				wGPINCheckCnt = 0;
				Gui_DC_Board_Connect = 1;	//
			}
		}
		else
		{
			wGPINCheckCnt = 0;
		}
	}
	else if(Gui_DC_Board_Connect == 1)
	{
		if(mGPIN_STATUS() == 1)
		{
			wGPINCheckCnt++;
			if(wGPINCheckCnt >= 5)
			{
				wGPINCheckCnt = 0;
				Gui_DC_Board_Connect = 0;	
				
				//if the user plug out the cable when the system is running
				if((wSysMode == cSoftStartMode) || (wSysMode == cNormalInvMode) )
				{
					FSuperFlag.BIT.FwFault = 1;
					Gui_wFaultCode = cBusVoltUnder;
				}
			}
		}
		else
		{
			wGPINCheckCnt = 0;
		}
	}
	
	if((FSuperFlag.BIT.FwAutoStartOnEn == 1) && (wSysMode == cStandByMode) && (wTurnOffByButton == 0))
	{
		if((Gui_wDcBusVoltAvg >= 3800) && ((Gui_wDcBusVoltAvg <= 4300)))
		{
			if(FSuperFlag.BIT.FwTurnOn == 0)
			{
				FSuperFlag.BIT.FwTurnOn = 1;
				FSuperFlag.BIT.FwAutoStartOnEn = 0;
			}
		}
	}	
}
//#ifdef FLASH
//----------------------------------------
void B4(void) //  Deal with the GUI command
//----------------------------------------
{
	//-----------------
	//the next time CpuTimer1 'counter' reaches Period value go to B1
	B_Task_Ptr = &B1;	
	//-----------------
	
	//Real time data sample command
	if(Gui_wLogEnable == 1)
	{
		
		if((Gui_CH1_ID > 0) && (Gui_CH1_ID < 32))
		{
			DLOG_4ch_i1Ptr = dwGraphIdTab[Gui_CH1_ID];
		}
		if((Gui_CH2_ID > 0) && (Gui_CH2_ID < 32))
		{
			DLOG_4ch_i2Ptr = dwGraphIdTab[Gui_CH2_ID];
		}
		if((Gui_CH3_ID > 0) && (Gui_CH3_ID < 32))
		{
			DLOG_4ch_i3Ptr = dwGraphIdTab[Gui_CH3_ID];
		}
		if((Gui_CH4_ID > 0) && (Gui_CH4_ID < 32))
		{
			DLOG_4ch_i4Ptr = dwGraphIdTab[Gui_CH4_ID];
		}	
		
		if(Gui_wSampleRate < DLOG_SIZE)
		{
			DLOG_4ch_PreScalar = Gui_wSampleRate;
		}
		
		if(Gui_wSampleLength <= DLOG_SIZE)
		{
			DLOG_4ch_Size = Gui_wSampleLength;
		}	
				
		DINT; 		
		wDataLogEnable = Gui_wLogEnable;
		Gui_wLogEnable = 0;
		Gui_wLogStatus = 1;
		EINT;
	}
	
	if(wSysMode == cStandByMode)
	{
		//Load Default Command
		if(Gui_wLoadDefault_Enable == 1)
		{
			//Calculate the parameter ralted to the Utility
			if(Gui_Utility_Freq == c50Hz)
			{
				wCyclePeriodCnt = 384;
		
				Gui_wFreq_High_Limit = 5500;		//55Hz
				Gui_wFreq_Low_Limit = 4500;			//45Hz
				Gui_wFreq_High = Gui_wFreq_High_Limit;		//55Hz
				Gui_wFreq_Low = Gui_wFreq_Low_Limit;			//45Hz	
				

			}
			else
			{
				wCyclePeriodCnt = 320;
		
				Gui_wFreq_High_Limit = 6600;		//66Hz
				Gui_wFreq_Low_Limit = 5400;			//54Hz
				
			}
			
			if(Gui_Utility_Volt == c120VAC)
			{
		
				Gui_wInvVolt_High_Limit = (Gui_Utility_Volt + c12VAC);		//132V
				Gui_wInvVolt_Low_Limit = (Gui_Utility_Volt - c12VAC);		//108V	
				Gui_wInvVolt_High = Gui_wInvVolt_High_Limit;		 
				Gui_wInvVolt_Low = Gui_wInvVolt_Low_Limit;		 	
		
			}
			else if(Gui_Utility_Volt == c220VAC)
			{
				Gui_wInvVolt_High_Limit = (Gui_Utility_Volt + c22VAC);		//132V
				Gui_wInvVolt_Low_Limit = (Gui_Utility_Volt - c22VAC);		//108V	
				Gui_wInvVolt_High = Gui_wInvVolt_High_Limit;		 
				Gui_wInvVolt_Low = Gui_wInvVolt_Low_Limit;		 	
			}
			
			Gui_wLoadDefault_Enable = 0;	
		}
		
		//Utiligy range setting command
		if(Gui_wSetting_Enable == 1)
		{
			Gui_wSetting_Enable = 0;
			
			Gui_wInvVolt_High_Limit = Gui_wInvVolt_High;		//132V
			Gui_wInvVolt_Low_Limit = Gui_wInvVolt_Low;		//108V
			Gui_wFreq_High_Limit = Gui_wFreq_High;		//55Hz
			Gui_wFreq_Low_Limit = Gui_wFreq_Low;			//45Hz		
		}
		
		//Utility Setting
		if(Gui_Utility_Setting_Enable != Gui_wUtility_Setting)
		{
			if(Gui_wUtility_Setting == 1)
			{
				Gui_Utility_Freq = c60Hz;
				Gui_Utility_Volt = c120VAC;
				wCyclePeriodCnt = 320;
				Gui_wInvVolt_High_Limit = (Gui_Utility_Volt + c12VAC);		
				Gui_wInvVolt_Low_Limit = (Gui_Utility_Volt - c12VAC);	
				Gui_wInvVolt_High = Gui_wInvVolt_High_Limit;
				Gui_wInvVolt_Low = Gui_wInvVolt_Low_Limit;
				Gui_wFreq_High_Limit = 6600;		//55Hz
				Gui_wFreq_Low_Limit = 5400;			//45Hz	
				Gui_wFreq_High = Gui_wFreq_High_Limit;
				Gui_wFreq_Low = Gui_wFreq_Low_Limit;
				Gui_Utility_Setting_Enable = Gui_wUtility_Setting;
				
				//Notch filter for 60Hz system, sample rate 416e-6s
				CNTL_2P2Z_CoefStruct3.b2 = _IQ26(0.9);
				CNTL_2P2Z_CoefStruct3.b1 = _IQ26(-1.712);
				CNTL_2P2Z_CoefStruct3.b0 = _IQ26(0.9);
				CNTL_2P2Z_CoefStruct3.a2= _IQ26(-0.8);
				CNTL_2P2Z_CoefStruct3.a1= _IQ26(1.712);
				CNTL_2P2Z_CoefStruct3.max = _IQ24(0.99);		// rad
				CNTL_2P2Z_CoefStruct3.min = _IQ24(-0.99);		// rad
				
				dwPll_Trace_Freq = _IQ20(376.9911);	 				//60Hz
			}
			else if(Gui_wUtility_Setting == 2)
			{
				Gui_Utility_Freq = c50Hz;
				Gui_Utility_Volt = c220VAC;
				wCyclePeriodCnt = 384;
				Gui_wInvVolt_High_Limit = (Gui_Utility_Volt + c12VAC);		
				Gui_wInvVolt_Low_Limit = (Gui_Utility_Volt - c12VAC);		
				Gui_wInvVolt_High = Gui_wInvVolt_High_Limit;
				Gui_wInvVolt_Low = Gui_wInvVolt_Low_Limit;
				Gui_wFreq_High_Limit = 5500;		//55Hz
				Gui_wFreq_Low_Limit = 4500;			//45Hz	
				Gui_wFreq_High = Gui_wFreq_High_Limit;
				Gui_wFreq_Low = Gui_wFreq_Low_Limit;
				Gui_Utility_Setting_Enable = Gui_wUtility_Setting;
				
				//Notch filter for 50Hz system, sample rate 416e-6s
				CNTL_2P2Z_CoefStruct3.b2 = _IQ26(0.9);
				CNTL_2P2Z_CoefStruct3.b1 = _IQ26(-1.739);
				CNTL_2P2Z_CoefStruct3.b0 = _IQ26(0.9);
				CNTL_2P2Z_CoefStruct3.a2= _IQ26(-0.8);
				CNTL_2P2Z_CoefStruct3.a1= _IQ26(1.739);
				CNTL_2P2Z_CoefStruct3.max = _IQ24(0.99);		// rad
				CNTL_2P2Z_CoefStruct3.min = _IQ24(-0.99);		// rad
				
				dwPll_Trace_Freq = _IQ20(314.2);	 				//50Hz
			}
		}
		
		//Enable or disable the Grid-Tie
		if(wGridTieEnable != Gui_GridTie_Enable)
		{
			DINT;
			wGridTieEnable = Gui_GridTie_Enable;
			EINT;
			
			if(wGridTieEnable == 0)
			{
				DINT;
				if(Gui_Utility_Freq == c50Hz)
				{
					dwPll_Trace_Freq = _IQ20(314.2);	 				//50Hz
				}
				else if(Gui_Utility_Freq == c60Hz)
				{
					dwPll_Trace_Freq = _IQ20(376.9911);	 				//60Hz
				}
				EINT;
			}
		}
	}
	
	//Turn off command
	if(Gui_wTurnOff == 1)
	{
		if((wSysMode == cSoftStartMode) || (wSysMode == cNormalInvMode))
		{
			//FSuperFlag.BIT.FwTurnOff = 1;
			//Send_Turn_Off_Command();		//Send the turn off command
			wSendTurnOffCmd = 1;
			if(Gui_DC_Board_Connect == 0)
			{
				FSuperFlag.BIT.FwTurnOff = 1;
			}
		}		
		Gui_wTurnOff = 0;
	}
	
	//Turn on command
	if(Gui_wTurnOn == 1)
	{
		Gui_wTurnOn = 0;
		
		if(wSysMode == cStandByMode)
		{
			if(FSuperFlag.BIT.FwWarning == 1)
			{
				if(wTurnOffByButton == 1)
				{
					wTurnOffByButton = 0;
				}
				FSuperFlag.BIT.FwWarning = 0;
				//wSendTurnOnCmd = 1;			
			}
			else if(FSuperFlag.BIT.FwTurnOn == 0)
			{
				//if(FSuperFlag.BIT.FwAutoStartOnEn == 0)
				if(Gui_DC_Board_Connect == 0)
				{
					FSuperFlag.BIT.FwTurnOn = 1;
					//wSendTurnOnCmd = 1;			//Test
					//Send_Turn_On_Command();	//Test
				}			
				else if(Gui_DC_Board_Connect == 1)
				{
					wSendTurnOnCmd = 1;	
					FSuperFlag.BIT.FwAutoStartOnEn = 1;
					//Send_Turn_On_Command();//add the system turn on code
				}	
			}
		}		
	}
	
	//////////////////////////////////////////////////////////////////
	//Lin communication
	//////////////////////////////////////////////////////////////////
	//DINT;
	//wNoSciRevIntCnt++;
	//EINT;
	//if(wNoSciRevIntCnt >= 2)
	//{
	//	DINT;
	//	wNoSciRevIntCnt = 2;
	//	EINT;
	//	Gui_DC_Board_Connect = 0;
	//}
	
	
	
	if(wSysMode == cStandByMode)
	{
		if(Gui_DC_Board_Connect == 1)
		{
			wVoltageLoopEnable = 1;
		}
		else
		{
			wVoltageLoopEnable = 0;
		}		
	}	
	if(wSendTurnOnCmd == 1)
	{
		Send_Turn_On_Command();	
	}
	else if(wSendTurnOffCmd == 1)
	{
		Send_Turn_Off_Command();
	}
	else
	{
		switch(countLin)
		{
			case 0: Inquiry_Panel_Voltage();
					countLin++;
					break;
			case 1: Inquiry_Panel_Current();
					countLin++;
					break;
			case 2: Inquiry_Boost_Voltage();
					countLin++;
					break;
			case 3: Inquiry_Panel_Output_Power();
					countLin++;
					break;
			case 4: Inquiry_MPPT_Status();
					countLin++;
					break;
			case 5: Send_Running_Mode();
					countLin++;
					break;
			case 6: Send_LLC_Output_Voltage();
					countLin = 0;
					break;
			default:countLin = 0;
					break;			
		}	
	}
	
}
//#endif //(FLASH)

//=================================================================================
//	C - TASKS
//=================================================================================

//--------------------------------- USER ------------------------------------------

//------------------------------------------------------
void C1(void) 	 
//------------------------------------------------------
{
if (INCR_BUILD == 2)
{
	//-----------------
	//the next time CpuTimer2 'counter' reaches Period value go to C2
	C_Task_Ptr = &C2;	
	//-----------------
}
else	C_Task_Ptr = &C1;
}

//----------------------------------------
void C2(void) 
//----------------------------------------
{
	//the next time CpuTimer2 'counter' reaches Period value go to C3
	C_Task_Ptr = &C3;	
	//-----------------
}


//-----------------------------------------
void C3(void) // Slew Rate ("Soft Start")
//-----------------------------------------
{
	//-----------------
	//the next time CpuTimer2 'counter' reaches Period value go to C1
	C_Task_Ptr = &C1;	
	//-----------------
}

#ifdef FLASH
//-----------------------------------------
void C4(void) //  SPARE
//-----------------------------------------
{

	//-----------------
	//the next time CpuTimer2 'counter' reaches Period value go to C1
	C_Task_Ptr = &C1;	
	//-----------------
}
#endif //(FLASH)

//----------------------------------------------------------------------
//Function name: sPowerOnMode
//Description: Deal with the power on affairs
//----------------------------------------------------------------------
void	sPowerOnMode(void)
{
	
	mFREE_RUN_OFF();
	mALL_PWM_OFF();	
	mOPRLY_OFF();

	wSysMode = cStandByMode;
	return;
}

//---------------------------------------------------------------------
//Function name: sStandByMode
//Description: The mode waiting for the turn on command
//---------------------------------------------------------------------
void	sStandByMode(void)
{
	if(FSuperFlag.BIT.FwFault == 1)
	{
		mALL_PWM_OFF();	
		mFREE_RUN_OFF();
		mOPRLY_OFF();
		wSysMode = cFaultMode;
		return;
	}
	//if((FSuperFlag.BIT.FwTurnOn == 1) && (FSuperFlag.BIT.FwTurnOnConditionOk == 1))
	if( (FSuperFlag.BIT.FwTurnOn == 1) && !Pvars.FaultLock )
	{
		wSysMode = cSoftStartMode;
		FSuperFlag.BIT.FwWarning = 0;
		FSuperFlag.BIT.FwTurnOn = 0;
		wStartCheckCnt = 0;
		return;
	}
	//else if((FSuperFlag.BIT.FwTurnOn == 1) && (FSuperFlag.BIT.FwTurnOnConditionOk == 0))
	//{
	//	wSysMode = cStandByMode;
	//	FSuperFlag.BIT.FwWarning = 1;
	//	FSuperFlag.BIT.FwTurnOn = 0;
	//	return;
	//}
	
}

void	sSoftStartMode(void)
{
	static int16 wTemp = 0;
	
	if(FSuperFlag.BIT.FwFault == 1)
	{
		mALL_PWM_OFF();	
		mFREE_RUN_OFF();
		mOPRLY_OFF();							
		wSysMode = cFaultMode;
		
		
		return;
	}
	
	if(FSuperFlag.BIT.FwTurnOff == 1)			//
	{
		FSuperFlag.BIT.FwTurnOff = 0;
		mALL_PWM_OFF();	
		mFREE_RUN_OFF();
		mOPRLY_OFF();	
		wSysMode = cStandByMode;
		wTurnOffByButton = 1;
		FSuperFlag.BIT.FwWarning = 1;
		wTemp = 0;
		return;
	}
	
	if(FSuperFlag.BIT.FwOpRlyStatus == 0)
	{
		wTemp = 0;
		mOPRLY_ON();	//Output relay switch to the main circuit side			
		dwBus_Voltage_Ref = Gui_Bus_Volt_Ref;
	}
	else	//The relay is on the main circuit side
	{
		wTemp++;
		
		if(wTemp < 2)		//40ms, waiting for the relay
		{
			return;
		}

		if(FSuperFlag.BIT.FwPwmStatus == 0)
		{
			//dwBus_Voltage_Ref = dwBus_Voltage_Fbk;
			mFREE_RUN_ON();
			DINT;
			DPL_Zero_Buff();
			EINT;
			Pvars.PWMonSyncFlag = 1;    // this forces the event to occur at zero crossing of the sine wave
			//wTemp = 0;
			return;			
		}		

		if( ((dwBus_Voltage_Fbk <= (Gui_Bus_Volt_Ref + cBus10V)) &&
			(dwBus_Voltage_Fbk > (Gui_Bus_Volt_Ref - cBus10V))) || Pvars.ForceAdvance)
		{
		    wSysMode = cNormalInvMode;
		    dwBus_Voltage_Ref = Gui_Bus_Volt_Ref;
			wTemp = 0;
			return;
		}
		if(dwBus_Voltage_Ref >= (Gui_Bus_Volt_Ref + cBus5V))
		{
			dwBus_Voltage_Ref -= cBus1V;		//Increase the BUS by 1V per 20ms
		}
		
		#if(Protect_Enable == 1)
		if(wTemp >= 1000)			// >10s
		{
			//wTemp = 500;		//For test
			mALL_PWM_OFF();	
			mFREE_RUN_OFF();
			asm(" nop");
			asm(" nop");
			asm(" nop");
			asm(" nop");
			mOPRLY_OFF();	
			wSysMode = cFaultMode;
			Gui_wFaultCode = cSoftStartFail;
			FSuperFlag.BIT.FwFault = 1;
			wSysMode = cStandByMode;
			wTemp = 0;
			return;
		}	
		#endif	
	}
}

void	sNormalInvMode(void)
{
	if((FSuperFlag.BIT.FwTurnOff == 1))
	{
		FSuperFlag.BIT.FwTurnOff = 0;
		mALL_PWM_OFF();	
		mFREE_RUN_OFF();
		mOPRLY_OFF();			
		wSysMode = cStandByMode;
		wTurnOffByButton = 1;
		FSuperFlag.BIT.FwWarning = 1;
		
		return;
	}

	if(FSuperFlag.BIT.FwFault == 1)
	{	
		mALL_PWM_OFF();	
		mFREE_RUN_OFF();
		mOPRLY_OFF();	
		wSysMode = cFaultMode;
		return;
	}
}

void	sFaultMode(void)
{
	if(FSuperFlag.BIT.FwClearFaultFlag == 1)
	{
		FSuperFlag.BIT.FwFault = 0;
		FSuperFlag.BIT.FwWarning = 0;
		FSuperFlag.BIT.FwClearFaultFlag = 0;
		wSysMode = cStandByMode;
		Gui_wFaultCode = 0;
		return;
	}
}

void	sSwKeyDet(void)
{
	static int16 wTemp = 0;

	if(wSysMode == cStandByMode)
	{
		if(mSWON == 0)
		{
			wTemp++;
			if(wTemp >= cSec1_0s)	//1s
			{
				if(FSuperFlag.BIT.FwWarning == 1)
				{
					if(wTurnOffByButton == 1)
					{
						wTurnOffByButton = 0;
					}
					FSuperFlag.BIT.FwWarning = 0;
					wTemp = 0;
					return;
				}
				else if(FSuperFlag.BIT.FwTurnOn == 0)
				{
					if(FSuperFlag.BIT.FwAutoStartOnEn == 0)
					{
						FSuperFlag.BIT.FwTurnOn = 1;
					}
					else
					{
						Send_Turn_On_Command();
					}				
					wTemp = 0;
					return;
				}
				else
				{
					wTemp = 0;
					return;
				}
			}
		}
		else
		{
			wTemp = 0;
			return;
		}
	}
	else if(wSysMode == cSoftStartMode)
	{
		//need to add the cancel soft start function
		if(mSWON == 0)
		{
			wTemp++;
			if(wTemp >= cSec1_0s)	//1s
			{
				if(FSuperFlag.BIT.FwTurnOff == 0)
				{
					FSuperFlag.BIT.FwTurnOff = 1;
					wTemp = 0;
					return;
				}
				else
				{
					wTemp = 0;
					return;
				}
			}
		}
		else
		{
			wTemp = 0;
			return;
		}
	}
	else if(wSysMode == cNormalInvMode)
	{
		if(mSWON == 0)
		{
			wTemp++;
			if(wTemp >= cSec1_0s)	//1s
			{
				if(FSuperFlag.BIT.FwTurnOff == 0)
				{
					FSuperFlag.BIT.FwTurnOff = 1;
					wTemp = 0;
					return;
				}
				else
				{
					wTemp = 0;
					return;
				}
			}
		}
		else
		{
			wTemp = 0;
			return;
		}
	}
	else if(wSysMode == cFaultMode)
	{
		//Clear fault operation
		if(mSWON == 0)
		{
			wTemp++;
			if(wTemp >= cSec1_0s)	//1s
			{
				if(FSuperFlag.BIT.FwClearFaultFlag == 0)
				{
					FSuperFlag.BIT.FwClearFaultFlag = 1;
					wTemp = 0;
					return;
				}
				else
				{
					wTemp = 0;
					return;
				}
			}
		}
		else
		{
			wTemp = 0;
			return;
		}
		
	}
}

//Execute per 1ms
void	sLedControl(void)
{
	static  int16 wTemp1 = 0;
	static  int16 wTemp2 = 0;

	//Mode LED
	if(wSysMode == cPowerOnMode)
	{
		mModeLed_ON();
		//mFaultLed_ON();
	}
	else if(wSysMode == cStandByMode)
	{
		wTemp1++;
		if(wTemp1 <= cSec0_6s)
		{
			mModeLed_ON();
			if(Pvars.Pstate==Locked){
			    mFaultLed_ON();
			}
		}
		else if((wTemp1 > cSec0_6s) && (wTemp1 < cSec1_2s))
		{
			mModeLed_OFF();
			mFaultLed_OFF();
		}
		else
		{
			wTemp1 = 0;
		}		
	}
	else if(wSysMode == cSoftStartMode)
	{
		wTemp1++;
		if(wTemp1 <= cSec0_25s)
		{
			mModeLed_ON();
		}
		else if((wTemp1 > cSec0_25s) && (wTemp1 < cSec0_5s))
		{
			mModeLed_OFF();
		}
		else
		{
			wTemp1 = 0;
		}		
	}
	else if(wSysMode == cNormalInvMode)
	{
		mModeLed_ON();
		wTemp1 = 0;
	}
	else
	{
		mModeLed_OFF();
		wTemp1 = 0;
	}

	//Fault LED
	if(FSuperFlag.BIT.FwFault == 1)
	{
		mFaultLed_ON();
		wTemp2 = 0;
	}
	else if(FSuperFlag.BIT.FwWarning == 1)
	{
		wTemp2++;
		if(wTemp2 <= cSec0_6s)
		{
			mFaultLed_OFF();
		}
		else if((wTemp2 > cSec0_6s) && (wTemp2 < cSec1_2s))
		{
			mFaultLed_ON();
		}
		else
		{
			wTemp2 = 0;
		}		
	}
	else
	{
//		mFaultLed_OFF();
	}
	
}

/*************************************************************************************
 * Function Name: ECap1_ISR
 * Description: The ECAP1 interrupt service routine used to detect the utility freq
 * **********************************************************************************/
interrupt void ECap1_ISR(void)
{
   long dwTemp;
   dwUtility_Freq_Cnt = ECap1Regs.CAP1;
   dwFreqTemp = dwUtility_Freq_Cnt - dwUtility_Freq_Cnt_0;
   dwUtility_Period = (dwUtility_Freq_Cnt - dwUtility_Freq_Cnt_0)>>5;
   dwUtility_Freq_Cnt_0 = dwUtility_Freq_Cnt;
   //wCycleFlag = 1;
   
   if((dwUtility_Period > c20HzCnt) || (dwUtility_Period < c100HzCnt))
   {
	   	ECap1Regs.ECCLR.bit.CEVT1 = 1;
		ECap1Regs.ECCLR.bit.INT = 1;
		ECap1Regs.ECCTL2.bit.REARM = 1;
		
	   // Acknowledge this interrupt to receive more interrupts from group 4
   		PieCtrlRegs.PIEACK.all = PIEACK_GROUP4;
   		return;
   }
   
   dwTemp = (cCntToFreq / dwUtility_Period);		//100*frequency
   Gui_wInvFreq = (int16)dwTemp;
   dwInv_Frequency_0 = (dwTemp * cFreqToQ24) >> 2;
   dwInv_Frequency = (dwInv_Frequency_0 + dwInv_Frequency)>>1;
   wNoZeroCrossCnt = 0;
   wLineCycleFlag = 1;		//Set the line cycle flag

   ECap1Regs.ECCLR.bit.CEVT1 = 1;
   ECap1Regs.ECCLR.bit.INT = 1;
   ECap1Regs.ECCTL2.bit.REARM = 1;

   // Acknowledge this interrupt to receive more interrupts from group 4
   PieCtrlRegs.PIEACK.all = PIEACK_GROUP4;
}

/*------------------------------------------------------------------------
 * Function Name: sRMSCal
 * Description: Calculate the RMS value of the AC sample
 * Parameter: dwSum --- The squre sum of the AC sample
 * 			  dwRatio -- Q24 * dwRatio = Real value
 * 			  wFormat -- The float point 
 * -----------------------------------------------------------------------*/
int16    sRMSCal(long dwSum,long dwRatio,int wFormat)
{
	_iq24 sqrt_in,sqrt_out;
	long  dwTemp;
	int16 wTemp;
	
	sqrt_in = (long)(dwSum/wSumCnt);		
	sqrt_out = _IQ24sqrt(sqrt_in);
	dwTemp = _IQ7mpyIQX(sqrt_out,24,dwRatio,18);
	wTemp = (int16)((long)dwTemp*wFormat>>7);
	return wTemp;
}

/*------------------------------------------------------------------------
 * Function Name:sAverageVoltCal
 * Description: Calculate the DC bus voltage average value.
 * 
 * -----------------------------------------------------------------------*/
 int16   sAverageVoltCal(long dwSum,long dwRatio,int wFormat)
 {
 	_iq21 ave_in;
	long  dwTemp;
	int16 wTemp;
	
	ave_in = (long)(dwSum/wSumCnt);		
	dwTemp = _IQ7mpyIQX(ave_in,21,dwRatio,18);
	wTemp = (int16)((long)dwTemp*wFormat>>7);
	return wTemp;
 }
 
 /*------------------------------------------------------------------------
 * Function Name:sActivePowerCal
 * Description: Calculate the active power
 * 
 * -----------------------------------------------------------------------*/
 int16   sActivePowerCal(long dwSum,long dwRatio,int wFormat)
 {
 	_iq24 ave_in;
	long  dwTemp;
	int16 wTemp;
	
	ave_in = (long)(dwSum/wSumCnt);		
	dwTemp = _IQ7mpyIQX(ave_in,24,dwRatio,10);
	wTemp = (int16)((long)dwTemp*wFormat>>7);
	return wTemp;
 }

#if 	(Protect_Enable == 1)
/*------------------------------------------------------------------------
 * Function Name:sOverRangeChk
 * Description: Check if the wValue is out of range
 * -----------------------------------------------------------------------*/
int16   sOverRangeChk(int16 wValue,int16 wHighLimit,int16 *wChkCnt,int16 wCntValue)
{
	int16 *pData;

	pData = wChkCnt;
		
	if(wValue >= wHighLimit)
	{
		(*pData)++;
		if((*pData) >= wCntValue)
		{
			(*pData) = wCntValue;
			return 1;
		}
		else
		{
			return 0;
		}
	}
	else
	{
	/*	if((*pData) > 0)
		{
			(*pData)--;
			return 1;
		}
		else
		{
			return 0;
		}	*/
		(*pData) = 0;
		return 0;
	}	
}

int16   sUnderRangeChk(int16 wValue,int16 wLowLimit,int16 *wChkCnt,int16 wCntValue)
{
	int16 *pData;

	pData = wChkCnt;
		
	if(wValue <= wLowLimit)
	{
		(*pData)++;
		if((*pData) >= wCntValue)
		{
			(*pData) = wCntValue;
			return 1;
		}
		else
		{
			return 0;
		}
	}
	else
	{
		/*
		if((*pData) > 0)
		{
			(*pData)--;
			return 1;
		}
		else
		{
			(*pData) = 0;
			return 0;
		}*/
		(*pData) = 0;
		return 0;
				
	}	
}

/*------------------------------------------------------------------------
 * Function Name:sShortCircuitChk
 * Description: Check if the output is short
 * -----------------------------------------------------------------------*/
int16   sShortCircuitChk(int16 wVolt,int16 wCurr,int16 *wChkCnt,int16 wCntValue)
{
	int16 *pData;
	pData = wChkCnt;
	
	if((wVolt <= 200) && (wCurr >= 1000))
	{
		(*pData)++;
		if((*pData) >= wCntValue)
		{
			(*pData) = wCntValue;
			return 1;
		}
		else
		{
			return 0;
		}
	}
	else
	{
		if((*pData) > 0)
		{
			(*pData)--;
			return 1;
		}
		else
		{
			(*pData) = 0;
			return 0;
		}	
	}
}

int16   sRushChk(int16 wValue,int16 wRushLimit,int16 *wChkCnt,int16 wCntValue)
{
	int16 *pData;

	pData = wChkCnt;
		
	if(wValue >= wRushLimit)
	{
		(*pData)++;
		if((*pData) >= wCntValue)
		{
			(*pData) = wCntValue;
			return 1;
		}
		else
		{
			return 0;
		}
	}
	else
	{
		(*pData) = 0;
		return 0;
				
	}	
}
#endif


void ProtectionPolling(void){
    int16 ACamps;
    Uint16 DCvolts;

    ACamps = AdcResult.ADCRESULT4 - 2048;
    if (ACamps < 0) ACamps *= -1;           // make positive if negative
    DCvolts = AdcResult.ADCRESULT3;

    if(ACamps > Pvars.AmpsMax) Pvars.AmpsMax = ACamps;
    else if(!Pvars.AmpsTimer--){
        Pvars.AmpsTimer = 3000;
        Pvars.AmpsMax--;            // start to decay the value;
    }

    Pvars.Amps = ACamps;        // for easy monitoring during debug;
    Pvars.Volts = DCvolts;      // for easy monitoring during debug

    switch(Pvars.Pstate){
    case Start:
        if( (DCvolts < Pvars.OverVoltageTrip) && ( (DCvolts > (Pvars.UnderVoltageTrip + Pvars.Hysteresis)) || (!Pvars.UnderVoltEnable) )   \
                && (ACamps < Pvars.OverCurrentTrip) )  {        // test for all conditions ok to start
            Pvars.Pstate = Protect;
            Pvars.FaultLock = 0;
            Pvars.ForceAdvance = 0;                 // allow natural advance if possible
            Pvars.DelayTimer = DELAY_COUNTER;
        }
        break;
    case Protect:
        if((DCvolts > Pvars.OverVoltageTrip) || ((EPwm1Regs.TZFLG.bit.OST || EPwm1Regs.TZFLG.bit.OST) && (wSysMode == cNormalInvMode))){    // TZFLG = current trip
            Pvars.FaultLock = 1;        //
            mALL_PWM_OFF();
            mFREE_RUN_OFF();
            mOPRLY_OFF();
            wSysMode = cPowerOnMode;
            Pvars.Pstate = Locked;
            Pvars.AmpOut = ACamps;
        }
        else if ( (DCvolts < Pvars.UnderVoltageTrip) && Pvars.UnderVoltEnable ) {       // if undervolt and undervolt enabled
            Pvars.FaultLock = 1;        //
            mALL_PWM_OFF();
            mFREE_RUN_OFF();
            mOPRLY_OFF();
            wSysMode = cPowerOnMode;
            Pvars.Pstate = Start;
        }
        else if(!FSuperFlag.BIT.FwPwmStatus) FSuperFlag.BIT.FwTurnOn = 1;
        else if( wSysMode != cNormalInvMode ){
            Pvars.ForceAdvance = 1;                     // force advance to normal mode if necessary to force red led to steady.
        }
        if(Pvars.DelayTimer) Pvars.DelayTimer--;          // count down the delay counter
        break;
    case Locked:
        // Power Cycling is required to exit this state.
        break;
    }
}

void MoveCurrentLimtBar(void){     // move the current limit bar based on the polarity
    Uint8 i;                        // use in the for loops
    EALLOW;

    EPwm1Regs.TZDCSEL.bit.DCAEVT1 = 0;      //     DCAH hi = DCAEVT1 high for EPWM1
    EPwm2Regs.TZDCSEL.bit.DCAEVT1 = 0;      //     DCAH hi = DCAEVT1 high for EPWM2
    if(PolarityBit){                // the new polarity is positive it the polarity bit was a zero
        Comp2Regs.DACVAL.bit.DACVAL  =  Pvars.DACzero + Pvars.DAChighSide;     // LOAD THE HIGH SIDE VALUE
        Comp2Regs.COMPCTL.bit.CMPINV = 0x0;                         // Output of the comparator is not inverted now
        for(i=0; i<8; i++){
            if(!Comp2Regs.COMPSTS.bit.COMPSTS) break;     // wait for the comparator to be ready
        }
    }
    else{
        Comp2Regs.DACVAL.bit.DACVAL  =  Pvars.DACzero - Pvars.DAClowSide;     // LOAD THE LOW SIDE VALUE
        Comp2Regs.COMPCTL.bit.CMPINV = 0x1;                         // Output of the comparator is inverted now
        for(i=0; i<8; i++){
            if(!Comp2Regs.COMPSTS.bit.COMPSTS) break;     // wait for the comparator to be ready
        }
    }
    EPwm1Regs.TZDCSEL.bit.DCAEVT1 = TZ_DCAH_HI;      //     DCAH hi = DCAEVT1 high for EPWM1
    EPwm2Regs.TZDCSEL.bit.DCAEVT1 = TZ_DCAH_HI;      //     DCAH hi = DCAEVT1 high for EPWM2
    EDIS;
}
